/**********************************
 CUSTOM CONTROL
 **********************************/
angular.module('controls').directive('kkCustomControl', ['$compile', '$injector', function($compile, $injector) {
	return {
		restrict: 'E',
		scope: {
			controlName: '=',
			kkModel: '=',
			kkConfig: '=',
			kkOption: '=',
			kkData: '=',
			kkDataParent: '='
		},
		template: '<div></div>',
		link: function(scope, elem, attrs) {
			// check if the directive we want to open exists
			if (!scope.controlName) {
				console.log('kkCustomControl: Angular Directive not found - No control name given. ', scope);
				elem.replaceWith($('<pre class="fs-095">NO DIRECTIVE GIVEN</pre>'));
			} else {
				var camelCasedControlName = scope.controlName.replace(/-([a-z])/g, function (g) { return g[1].toUpperCase(); }); // transform my-control-name to myControlName
				camelCasedControlName = camelCasedControlName.charAt(0).toUpperCase() + camelCasedControlName.slice(1); // uppercase the first letter: myControlName - MyControlName
				
				if (!$injector.has('control' + camelCasedControlName + 'Directive')) {
					elem.replaceWith($('<div class="alert alert-danger fs-095"><strong>Oups</strong><br/>Es ist ein Fehler beim Laden des Drop Controls "control' + camelCasedControlName + '" aufgetreten. Bitte lade die Seite neu und versuche es erneut.</div>'));
					console.log('kkCustomControl: Angular Directive not found:' + 'control' + camelCasedControlName);
				} else {
					var control = '<control-' + scope.controlName + ' kk-model="kkModel" kk-config="kkConfig" kk-option="kkOption" kk-data="kkData" kk-data-parent="kkDataParent"></control-' + scope.controlName + '>';
					var jControl = $compile(control)(scope);
					elem.replaceWith(jControl);
				}
			}
		}
	};
}]);


/**********************************
 TEXT
 **********************************/
angular.module('controls').directive('controlText', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div ng-class="{\'input-append\' : kkOption.append}">\
				<input spellcheck="false" type="text" ng-model="kkModel" ng-model-options="{updateOn: \'default blur\', debounce: { default: 500, blur: 0 }}" placeholder="{{kkOption.description}}"/>\
				<span class="add-on" kk-if="kkOption.append">{{kkOption.append}}</span>\
			</div>\
		',
		link: function(scope, elem, attrs) {

		}
	};
}]);

/**********************************
 TEXTAREA
 **********************************/
angular.module('controls').directive('controlTextarea', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<textarea placeholder="{{kkOption.description}}" ng-model="kkModel" ng-model-options="{updateOn: \'default blur\', debounce: { default: 500, blur: 0 }}" name="{{kkOption.property}}"></textarea>\
		',
		link: function(scope, elem, attrs) {

		}
	};
}]);

/**********************************
 CHECKBOX
 **********************************/
angular.module('controls').directive('controlCheckbox', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<input type="checkbox" ng-model="kkModel" name="{{kkOption.property}}"/>\
		',
		link: function(scope, elem, attrs) {
		}
	};
}]);

/**********************************
 UPLOAD
 **********************************/
angular.module('controls').directive('controlUpload', ['core', 'media', 'message', function(core, media, message) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="frame">\
				<div class="kk-uploader">\
					<div class="uploader"></div>\
					<div class="uploader-buttons">\
						<div class="btn btn-media trigger-tooltip" data-original-title="Aus Mediendateien w&auml;hlen" ng-click="selectFromGallery(false)">\
							<i class="icon-picture"></i>\
						</div>\
						<div class="btn trigger-tooltip" data-original-title="URL zeigen/editieren" ng-click="toggleUrl()">\
							<i class="icon-map-marker"></i>\
						</div>\
					</div>\
					<input placeholder="Url" ng-show="showUrl" type="text" ng-model="kkModel"/>\
				</div>\
				<a class="thumbnail" kk-if="kkModel">\
					<img ng-src="{{kkModel}}" style="max-width: 100%;max-height: 100px;" ng-click="selectFromGallery(true)"></img>\
				</a>\
			</div>\
		',
		link: function(scope, elem, attrs) {
			scope.showUrl = false;

			scope.toggleUrl = function() {
				scope.showUrl = !scope.showUrl;
			};

			scope.selectFromGallery = function(activate) {
				if (activate && angular.isString(scope.kkModel) && scope.kkModel.length > 0) {
					media.select(function(res) {
						scope.kkModel = res.path;
					}, scope.kkModel);
				} else {
					media.select(function(res) {
						scope.kkModel = res.path;
					});
				}
			};

			if (!scope.kkModel) {
				scope.kkModel = "";
			}

			var uploader = new qq.FileUploader({
				element: $(elem).find('.uploader')[0],
				inputName: 'kk_dropper_media_upload',
				uploadButtonText: 'Datei hochladen',
				cancelButtonText: 'Abbrechen',
				failUploadText: 'Upload fehlgeschlagen',
				dragText: 'Ziehe Dateien zum Upload hier hin!',
				action: core.uploadURL,
				customHeaders: {
					'X-KK-Dropper-Admin-Pass': core.adminSecret
				},
				onComplete: function(id, fileName, responseJSON) {
					if (!responseJSON.error) {
						$('.qq-upload-list', elem).empty();
						scope.$apply(function() {
							console.log('upload complete: ', responseJSON);
							scope.kkModel = core.relativeShopURL + 'bilder/kk_dropper_uploads/' + responseJSON.filename;
						});
					}
				},
				onError: function(id, fileName, err) {
					message.error("Datei konnte nicht hochgeladen werden!", err);
				}
			});
		}
	};
}]);

/**********************************
 HTML
 **********************************/
angular.module('controls').directive('controlHtml', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div>\
				<div ace="drop" ng-model="kkModel" ng-model-options="{updateOn: \'default blur\', debounce: { default: 500, blur: 0 }}" kk-drop-droppable>\
					<textarea></textarea>\
				</div>\
			</div>\
		',
		link: function(scope, elem, attrs) {
		}
	};
}]);

/**********************************
 LINK
 **********************************/
angular.module('controls').directive('controlLink', ['modal', 'datasource', function(modal, datasource) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="btn" ng-click="showURLModal()" style="white-space: nowrap"><i class="icon-link"></i> <small kk-if="!buttonLabel">Link w&auml;hlen</small><small kk-if="buttonLabel"><strong>{{truncate(buttonLabel.label, 30)}}</strong><span kk-if="kkModel.title.length"> &quot;{{truncate(kkModel.title, 20)}}&quot;</span> ({{buttonLabel.type}})</small><i class="icon-remove-sign fadebtn" style="margin-left: 15px; margin-right: -5px;" title="Link entfernen" ng-click="unsetLink($event)" kk-if="buttonLabel"></i></div>\
		',
		link: function(scope, elem, attrs) {
			
			scope.typeConfig = {
				name: "Typ",
				description: "Wähle, woher der Link kommen soll",
				type: "switch",
				property: "type",
				options: [
					{ label: "Manueller Link", value: "Manual" },
					{ label: "Kategorie", value: "Categories" },
					{ label: "Hersteller", value: "Brands" },
					{ label: "CMS Seite", value: "Link" },
					{ label: "Artikel", value: "Articles" }
				]
			};

			scope.urlPickerConfig = [
				{
					name: "URL",
					description: "Linkpfad",
					type: "text",
					property: "ManualId",
					condition: "drop.type === 'Manual'"
				},
				{
					name: "Artikel",
					description: "Artikellink",
					type: "select",
					property: "ArticlesId",
					condition: "drop.type === 'Articles'",
					options: "Articles"
				},
				{
					name: "Kategorie",
					description: "Kategorielink",
					type: "select",
					property: "CategoriesId",
					condition: "drop.type === 'Categories'",
					options: "Categories"
				},
				{
					name: "CMS Seite",
					description: "Seitenlink",
					type: "select",
					property: "LinkId",
					condition: "drop.type === 'Link'",
					options: "Link"
				},
				{
					name: "Hersteller",
					description: "Herstellerlink",
					type: "select",
					property: "BrandsId",
					condition: "drop.type === 'Brands'",
					options: "Brands"
				},
				{
					name: "Linktitel",
					append: "optional",
					description: "Der Titel des Links",
					type: "text",
					property: "title"
				},
				{
					name: "Neuer Tab?",
					description: "Soll der Link in einem anderen Tab geöffnet werden (benötigt Javascript)?",
					options: [
						{ label: 'Ja', value: true },
						{ label: 'Nein', value: false }
					],
					type: "switch",
					property: "openExternal"
				}
			];

			scope.calculateButtonLabel = function(setting) {
				if (!setting || !setting.type) {
					scope.buttonLabel = false;
					return;	
				}
				// manual link?
				if (setting.type === 'Manual') {
					scope.buttonLabel = {
						label: setting.value,
						type: "Manueller Link"
					};
					return;
				}

				// loop through pickerConfig to get the correct datasource
				for(var i=0; i<scope.urlPickerConfig.length; i++) {
					var config = scope.urlPickerConfig[i];
					if (config.property === setting.type + 'Id' && angular.isString(config.options)) {

						// set loading state
						scope.buttonLabel = {
							label: "",
							type: "Lade..."
						};

						// resolve the datasource
						datasource.get(config.options, function(res) {
							for(var j=0; j<res.length; j++) {
								if (res[j].value === setting.value) {
									scope.buttonLabel = {
										label: res[j].label,
										type: config.name
									};
									scope.$apply();
									return;
								}
							}
							scope.buttonLabel = false;
						});
						break;
					}
				}
			};

			scope.showURLModal = function() {
				modal.show('UrlPickerModal', function(res) {
					if (typeof(res) !== "undefined") {
						scope.kkModel = res;
					}
				}, 
				{
					"urlPickerModel": scope.kkModel,
					"urlPickerConfig": scope.urlPickerConfig,
					"typeConfig": scope.typeConfig
				});
			};

			scope.unsetLink = function(e) {
				e.stopPropagation();
				scope.kkModel = null;
			};

			scope.truncate = function(text, length) {
				if (text) {
					return text.length > length ? text.substr(0, length) + '...' : text;
				}
				return "";
			};
			
			// initialize
			scope.buttonLabel = false;

			// watch the model for changes to update the label of the button
			scope.$watch('kkModel', function() {
				scope.calculateButtonLabel(scope.kkModel);
			});

		}
	};
}]);

/**********************************
 DATE
 **********************************/
angular.module('controls').directive('controlDate', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="datepicker" ng-model="kkModel" ui-date ui-date-format="kkOption.format"></div>\
		',
		link: function(scope, elem, attrs) {
			// set today as default, when nothing is set yet
			if (!scope.kkModel) {
				var dateFormat = scope.$eval(attrs.uiDateFormat) || 'mm/dd/yy';
				scope.kkModel = jQuery.datepicker.formatDate(dateFormat, new Date());
			}
		}
	};
}]);


/**********************************
 TIME
 **********************************/
angular.module('controls').directive('controlTime', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="control-time">\
				<div class="input-append">\
					<input type="text" ng-model="kkModel" datetime="HH:mm" datetime-model="HH:mm"/>\
					<span class="add-on"><i class="icon-time"></i> Uhr</span>\
				</div>\
			</div>\
		',
		link: function(scope, elem, attrs) {
		}
	};
}]);


/**********************************
 DROP
 **********************************/
angular.module('controls').directive('controlDrop', ['core', '$rootScope', function(core, $rootScope) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",	// data object (e.g. the drop itself)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="kk-drop-picker">\
				<control-select class="drop-select" kk-model="kkModel" kk-option="dropsOption"></control-select>\
				<span>\
					<div class="btn trigger-tooltip drop-select-add" data-toggle="dropdown" data-original-title="Neues Drop erstellen"><i class="icon-plus icon-small"></i>\
					</div>\
					<ul class="dropdown-menu nav" style="max-height: 400px; overflow-y: auto;">\
						<li ng-repeat="(templateKey, template) in templates" ng-click="rootScope.addAndLinkDrop(templateKey, kkData, kkOption.property)">\
							<a><small>{{template.name}}</small></a>\
						</li>\
					</ul>\
				</span>\
			</div>\
		',
		link: function(scope, elem, attrs) {
			scope.templates = core.templates;
			scope.rootScope = $rootScope;

			scope.dropsOption = {
				"type": "select",
				"options": $rootScope.settings.drops,
				"optionLabel": "_internal.name",
				"optionValue": "_internal.guid"
			};
		}
	};
}]);


/**
 * Generic Link Function for Option direcitves
 */
var kkOptionDirectiveLinkFunction = function(scope, elem, attrs, datasource, core, supportsSearch) {

	// default to no search support
	supportsSearch = supportsSearch || false;

	// check if the option itself changes, if so update scope bindings
	scope.$watch('kkOption', function() {
		if (!scope || !scope.kkOption) return;
		scope.givenOptions = scope.kkOption.options || [];
		scope.labelProperty = scope.kkOption.optionLabel || "label";
		scope.valueProperty = scope.kkOption.optionValue || "value";
		scope.imageProperty = scope.kkOption.optionImage || "image";
		scope.searchProperty = scope.kkOption.optionSearch || "search";
		scope.imageRoot = scope.kkOption.optionImageRoot || "drop";
		scope.detailProperty = scope.kkOption.detailProperty || "detail";
		scope.supportsSearch = supportsSearch;
	}, true);


	// update options whenever kkOptions has changed
	scope.$watch('givenOptions', function() {
		
		// options from another drop property (e.g. "drop.tags")?
		if (angular.isString(scope.givenOptions) && scope.givenOptions.indexOf('.') > 0) {

			scope.optionSource = 'internal';
			
			var varParts = scope.givenOptions.split('.');
			switch(varParts[0]) {
				case 'drop':
					if (scope.$root && scope.$root.activeDrop) scope.options = scope.$root.activeDrop[varParts[1]] || [];
				break;
				case 'self':
					scope.options = scope.kkData[varParts[1]] || [];
				break;
				case 'parent':
					if (scope.kkDataParent) scope.options = scope.kkDataParent[varParts[1]] || [];
				break;
			}
		}
		
		// datasource name (e.g. "Articles")?
		else if (angular.isString(scope.givenOptions) && scope.givenOptions.length > 0) {

			scope.optionSource = 'datasource';

			// check if the model changes... if so we have to update our options 
			// array via ajax to resolve to the correct data objects!
			if (!supportsSearch) {

				// lets check if we can find a datasource with the given name, if so, fetch data and apply results
				var realModel = scope.kkModel;
				datasource.get(scope.givenOptions, function(res){
					scope.$apply(function(){
						scope.options = res;
						scope.setModel(realModel);
					});
				});


			} else {
				scope.options = [];
			}

		}
		
		// direct options array?
		else if (angular.isArray(scope.givenOptions)) {

			scope.optionSource = 'direct';

			// options array was given
			scope.options = scope.givenOptions;
		}

	}, true);

	/**
	 * sets the kkModel value
	 */
	scope.setModel = function(value) {
		if (scope.kkModel !== value) {
			scope.kkModel = value;
		}
	};

	/**
	 * evaluate a path inside a given option... so something like option.x.name with path "x.name"
	 */
	scope.evalValue = function(option, path) {
		// option: { a: { b: 3 }}
		// path: a.b

		if (!path || path.length == 0 || !option) {
			return;
		}

		var parts = path.split('.');
		var current = option;
		for(var i=0; i<parts.length; i++) {
			if (typeof(current[parts[i]]) !== 'undefined') {
				current = current[parts[i]];
			} else {
				return;
			}
		}
		return current;
	};

	/**
	 * Transforms an image path depending on the given imageRoot option
	 */
	scope.transformImagePath = function(path) {
		if (!core) {
			return path;
		}

		switch(scope.imageRoot) {
			case "drop":
				return core.dropPathURL + scope.kkData._internal.dropId + "/" + path;
			break;

			case "drops":
				return core.dropPathURL + path;
			break;

			case "shop":
				return core.relativeShopURL + path;
			break;

			case "absolute":
				return path;
			break;
			
			case "admin":
				return core.adminURL + path;
			break;
			default:
				return path;
			break;
		}
	};
};

/**********************************
 SELECT
 **********************************/
angular.module('controls').directive('controlSelect', ['datasource', 'core', function(datasource, core) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",		// the configuration for this control
			kkModel: "=",		// model value to bind to
			kkData: "=",		// data object (e.g. the drop itself under which the model is bound)
			kkDataParent: "=",	// data object parent (e.g. an object, under which the the kkData is bound)
			kkConfig: "="		// whole configuration object
			
		},
		template: '<input type="hidden" ui-select2 ng-model="kkModel"></input>',
		link: function(scope, elem, attrs) { kkOptionDirectiveLinkFunction(scope, elem, attrs, datasource, core, true); }
	};
}]);

/**********************************
 MULTISELECT
 **********************************/
angular.module('controls').directive('controlMultiselect', ['datasource', 'core', function(datasource, core) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",		// data object (e.g. the drop itself under which the model is bound)
			kkDataParent: "=",	// data object parent (e.g. an object, under which the the kkData is bound)
			kkConfig: "="	// whole configuration object
		},
		template: '<input multiple type="hidden" ui-select2 ng-model="kkModel"></input>',
		link: function(scope, elem, attrs) { 
			kkOptionDirectiveLinkFunction(scope, elem, attrs, datasource, core, true);
		}
	};
}]);

/**********************************
 SWITCH
 **********************************/
angular.module('controls').directive('controlSwitch', ['datasource', 'core', function(datasource, core) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: "=",	// the configuration for this control
			kkModel: "=",	// model value to bind to
			kkData: "=",		// data object (e.g. the drop itself under which the model is bound)
			kkDataParent: "=",	// data object parent (e.g. an object, under which the the kkData is bound)
			kkConfig: "="	// whole configuration object
		},
		template: '\
			<div class="btn-group">\
				<div ng-repeat="option in options" ng-click="setModel(evalValue(option,valueProperty))" class="btn btn-mini" title="{{evalValue(option,labelProperty)}}" ng-class="{\'active\': evalValue(option,valueProperty) == kkModel}">\
					<img kk-if="evalValue(option,imageProperty)" ng-src="evalValue(option,imageProperty)"/>\
					<span kk-if="!evalValue(option,imageProperty)">\
						<span kk-if="evalValue(option,labelProperty) == undefined || evalValue(option, labelProperty) == \'\'" class="kk-label-empty">\
							(leer)\
						</span>\
						{{evalValue(option,labelProperty)}}\
					</span>\
				</div>\
			</div>',
		link: function(scope, elem, attrs) { kkOptionDirectiveLinkFunction(scope, elem, attrs, datasource, core, false); }
	};
}]);

/**********************************
 IMAGE PICKER
 **********************************/
angular.module('controls').directive('controlImagepicker', ['$rootScope', 'core', 'datasource', function($rootScope, core, datasource) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkModel: '=',
			kkOption: '=',
			kkData: "=",		// data object (e.g. the drop itself under which the model is bound)
			kkDataParent: "="	// data object parent (e.g. an object, under which the the kkData is bound)
		},
		template:
			'\
			<div class="image-picker-container" ng-click="$event.stopPropagation()">\
				<div class="btn btn-small image-picker-toggle-popover" ng-class="{active: popoverActive}" ng-click="togglePopover()">\
					<i class="icon-edit"></i>&nbsp;&nbsp;{{getActiveOptionLabel()}}\
				</div>\
				<div class="image-picker-options popover bottom" ng-show="popoverActive" style="width: {{kkOption.width}}px;">\
					<div class="arrow"></div>\
					<div class="popover-title"><span ng-click="togglePopover()" class="pull-right close">&times;</span>{{kkOption.name}}<br/><span class="image-picker-value">{{optionHint}}</span></div>\
					<div class="popover-content" ng-mouseleave="setOptionHint()">\
						<div ng-repeat="option in options" class="image-picker-option" ng-class="{active: isActiveOption(option)}" ng-click="setOption(option)" ng-mouseenter="setOptionHint(option)">\
							<img ng-src="{{transformImagePath(evalValue(option, imageProperty))}}"/>\
							<div class="image-picker-option-active" kk-if="isActiveOption(option)">\
								<i class="icon-ok"></i>\
							</div>\
						</div>\
					</div>\
				</div>\
			</div>\
			',

		link: function(scope, elem, attrs) {

			// run our generic link function that creates an "option" array
			kkOptionDirectiveLinkFunction(scope, elem, attrs, datasource, core, false);

			/**
			 * Sets a given option as value
			 */
			scope.setOption = function(option) {
				// set the value
				scope.setModel(scope.evalValue(option, scope.valueProperty));
				// deactivate the popup
				scope.popoverActive = false;
			};

			/**
			 *	Checks if the given option is the active one
			 */
			scope.isActiveOption = function(option) {
				return (scope.kkModel === scope.evalValue(option, scope.valueProperty));
			};

			/**
			 * Gets the label of the active option
			 */
			scope.getActiveOptionLabel = function() {
				if (!scope.options) {
					return "Keine Auswahl";
				}

				for(var i=0; i<scope.options.length; i++) {
					var option = scope.options[i];
					if (option && scope.evalValue(option, scope.valueProperty) === scope.kkModel) {
						return scope.evalValue(option, scope.labelProperty);
					}
				}
				return "Keine Auswahl";
			};

			/**
			 *	Sets the hint of the popover (hovered Option)
			 */
			scope.setOptionHint = function(option) {
				if (!option) {
					scope.optionHint = scope.getActiveOptionLabel();
				}
				else {
					scope.optionHint = scope.evalValue(option, scope.labelProperty);
				}
			};

			/**
			 * Toggles the Popover to be shown or hidden
			 */
			scope.togglePopover = function() {
				scope.popoverActive = !scope.popoverActive;
			};


			/**
			 * Closes the popover when the body is clicked
			 */
			var closeOnBodyClick = function() {
				scope.$apply(function() {
					scope.popoverActive = false;
				});
			};

			$('#content').on('click.controlImagePicker', closeOnBodyClick);
			
			// deregister clicker
			scope.$on('$destroy', function() {
				$('#content').off('click.controlImagePicker', closeOnBodyClick);
			});

			// init values
			scope.popoverActive = false;
			scope.optionHint = scope.getActiveOptionLabel();
		}
	};
}]);

/**********************************
 COLOR PICKER
 **********************************/
angular.module('controls').directive('controlColor', [function() {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkModel: '=',
			kkOption: '=',
			kkData: '='
		},
		template: '<div class="input-append" style="position:relative"><input colorpicker="rgba" colorpicker-parent="true" type="text" ng-model="kkModel" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"/><span class="add-on" kk-if="kkModel" ng-style="getPickerStyle()"></span></div></div>',

		link: function(scope, elem, attrs) {
			scope.getPickerStyle = function() {
				return {
					'background-color': scope.kkModel,
					'width': '40px',
					'border': '3px solid white',
					'border-radius': '4px',
					'box-shadow': 'inset 0 0 0 1px rgba(0,0,0,0.15)',
					'pointer-events': 'none'
				};
			};
		}
	};
}]);

/**********************************
 PROPERTY GRID
 
 Creates a dynamic Property Grid
 Usage:
 <kk-propertygrid kk-model="myobject" kk-config="myconfigtemplate"></kk-propertygrid>
 **********************************/
angular.module('controls').directive('kkPropertygrid', ['environment', 'core', function(environment, core) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkModel: '=',
			kkConfig: '=',
			kkModelScope: '='
		},
		templateUrl: core.adminURL + 'js/controls/propertygrid.html.js?' + new Date().getTime(),
		controller: function($scope, $element) {
			$scope.environment = environment;

			$scope.$watch('kkConfig', function() {
				setTimeout(function() {
					$element.find('.trigger-tooltip').tooltip({placement: 'right'});
				}, 400);
			});
			$scope.uploadUrl = core.uploadURL;

			$scope.canShowProperty = function(option, model, modelParent) {
				// if a condition is given, evaluate it as well
				if (angular.isString(option.condition) && option.condition !== "" && model) {
					var dropItem = (modelParent ? model : null);
					var drop = (modelParent ? modelParent : model);

					var self = model;
					var parent = (modelParent ? modelParent : null);
					
					var checkFn = "(" + option.condition + ")";
					return eval(checkFn);
				}
				return true;
			};

			$scope.evalProp = function(option, model) {
				if (option.multilanguage === true || option.multilanguage === "true") {
					// multilanguage option
					if (!model[option.property]) {
						model[option.property] = {_multilanguage: true};
					}
					else if (model[option.property] && typeof(model[option.property]) !== 'object') {
						console.log('Property "' + option.property + '" should be multilanguage but is not. Converting to "' + $scope.environment.languageDefault.cISO + '"');
						var valueUnlanguaged = model[option.property];
						model[option.property] = {_multilanguage: true};
						model[option.property][$scope.environment.languageDefault.cISO] = valueUnlanguaged;
					}
				} else {
					// no multilanguage
					if (typeof(model[option.property]) === 'undefined') {
						// model[option.property] = "";
					}
					else if (model[option.property] && typeof(model[option.property]) === 'object' && model[option.property]._multilanguage === true) {
						console.log('Property "' + option.property + '" is multilanguage but should not be. Converting to simple string using default language "' + $scope.environment.languageDefault.cISO + '"', "Property Value:", model[option.property], "Option:", option);
						if (typeof model[option.property][$scope.environment.languageDefault.cISO] !== 'undefined') {
							model[option.property] = model[option.property][$scope.environment.languageDefault.cISO];
						} else {
							model[option.property] = "";
						}
						console.log("New Property Value:", model[option.property]);
					}
				}
				// return true;
			};
			
			$scope.updateSections = function() {
				if (!angular.isArray($scope.kkConfig)) return {};
				$scope.sections = {};

				for(var i in $scope.kkConfig) {
					var itemCfg = $scope.kkConfig[i];

					if (itemCfg && itemCfg.section && angular.isString(itemCfg.section)) {
						if (!$scope.sections[itemCfg.section]) {
							$scope.sections[itemCfg.section] = [];
						}
						$scope.sections[itemCfg.section].push(itemCfg);
					} else {
						if (!$scope.sections['Allgemein']) {
							$scope.sections['Allgemein'] = [];
						}
						$scope.sections['Allgemein'].push(itemCfg);
					}
				}
			};
			
			$scope.isSectionVisible = function(options) {
				for (var i=0; i<options.length; i++) {
					// check option visibility
					if ($scope.canShowProperty(options[i], $scope.kkModel, $scope.kkModelScope)) return true;
				}
				return false;
			};
			
			$scope.hasOnlyDefaultSection = function() {
				var defaultFound = false;
				for(var section in $scope.sections) {
					if (section === 'Allgemein') { 
						defaultFound = true;
					} else {
						return false;
					}
				}
				return defaultFound;
			};

			$scope.setActiveSection = function(section) {
				$scope.activeSection = section;
			};

			$scope.isActiveSection = function(section) {
				return $scope.activeSection == section;
			};
			
			$scope.activateDefaultSection = function() {
				if (!$scope.sections) return;
				// active the first section we find
				for(var section in $scope.sections) {
					$scope.setActiveSection(section);
					return;
				}
			};

			$scope.updateSections();
			$scope.activateDefaultSection();
		}
	};
}]);

/**********************************
 UI SELECT 2
 **********************************/
angular.module('controls').directive('uiSelect2', ['$http', 'datasource', function ($http, datasource) {

	var options = {placeholder: 'Auswahl treffen', allowClear: true};

	return {
		require: '?ngModel',
		compile: function (tElm, tAttrs) {
			
			var isMultiple = (tAttrs.multiple !== undefined);
			var isInitialized = false;
			var xhrRequest = null;
			var searchResults = [];

			return function (scope, elm, attrs, controller) {

				// instance-specific options
				var opts = angular.extend({}, options, scope.$eval(attrs.uiSelect2));

				opts.minimumInputLength = 0;
				opts.multiple = isMultiple ? true : false;
				opts.formatResult = function (item) {
					var value = scope.evalValue(item, scope.labelProperty);
					var details = scope.evalValue(item, scope.detailProperty);
					return value + (details ? '<div class="select2-item-details">' + details + '</div>' : '');
				};
				opts.formatSelection = function (item) { return scope.evalValue(item, scope.labelProperty); };
				opts.formatNoMatches = function () { return "Nichts gefunden"; };
				opts.formatInputTooShort = function (input, min) { var n = min - input.length; return "Bitte gib " + n + " oder mehr Zeichen ein"; };
				opts.formatLoadMore = function (pageNumber) { return "Lade..."; };
				opts.formatSearching = function () { return "Suche..."; };
				opts.id = function (item) { return scope.evalValue(item, scope.valueProperty); };
				
				var udpateOpts = function() {

					// options are given... respect that and not do a live search query
					if (scope.optionSource !== 'datasource') {
	
						opts.data = function() {
							return {
								results: scope.options,
								text: function(item) {
									if (scope.searchProperty) {
										var textVal = scope.evalValue(item, scope.searchProperty);
										if (textVal) return textVal;
									} 
									return scope.evalValue(item, scope.labelProperty);
								},
								id: function(item) {
									return scope.evalValue(item, scope.valueProperty);
								}
							};
						};
						
					} 
					
					// live query datasources!
					else {
						
						opts.query = function (query) {
							
							if (!angular.isString(scope.givenOptions)) return;
		
							// abort other xhr requests
							if (xhrRequest) xhrRequest.abort();
		
							xhrRequest = datasource.search(scope.givenOptions, query.term, query.page - 1, function(res) {
								searchResults = res.results || [];
		
								query.callback({
									results: searchResults,
									more: res.more || false
								});
							});
		
						};
	
	
						opts.initSelection = function(elem, callback) {
							
							// check if we have everything given to init our selection
							if (!scope.valueProperty || !scope.options || !scope.givenOptions) return;
		
							// get the keys to init the selection for
							var keys = scope.kkModel;
		
							var isMultiple = angular.isArray(keys);
		
							// if this is a single value, make it an array...
							if (angular.isDefined(keys) && !isMultiple) {
								keys = [keys];
							}
							
							// resolve the keys
							datasource.resolve(scope.givenOptions, scope.valueProperty, keys, function(results) {
								if (!results || !results.length) return;
								return callback(isMultiple ? results : results[0]);
							});
		
						};
					}
				};
				
				// update opts first run
				udpateOpts();

				if (controller) {
					// Watch the model for programmatic changes
					controller.$render = function () {
						if (isMultiple && typeof(controller.$modelValue) === 'undefined') {
							elm.select2('val', []);
						} else {
							elm.select2('val', controller.$modelValue);
						}
					};

					// Set the view and model value and update the angular template manually for the ajax/multiple select2.
					elm.bind("change", function () {
						scope.$apply(function () {

							var selectedData = elm.select2('data');
							var modelData;

							// transform the data into something we want to set to our model!
							if (angular.isArray(selectedData)) {
								modelData = [];
								for(var i=0; i<selectedData.length; i++) {
									modelData.push(scope.evalValue(selectedData[i], scope.valueProperty));
								}
							}
							else if (angular.isObject(selectedData)) {
								modelData = scope.evalValue(selectedData, scope.valueProperty);
							}
							else {
								modelData = null;
							}

							controller.$setViewValue(modelData);
							elm.select2('val', controller.$viewValue);
						});
					});
				}

				attrs.$observe('disabled', function (value) {
					elm.select2(value && 'disable' || 'enable');
				});

				// when the option set changes, update the default value
				scope.$watch("options", function(newVal, oldVal) {

					udpateOpts();
					
					// init select2 with the given options
					if (scope.options && (!isInitialized || scope.optionSource !== 'datasource')) {

						// initialize select2
						elm.select2(opts);
						
						// make it sortable
						if (isMultiple) {
							elm.parent().find("ul.select2-choices").sortable({
								containment: 'parent',
								start: function() { elm.select2("onSortStart"); },
								update: function() { elm.select2("onSortEnd"); }
							});
						}

						isInitialized = true;
					}

					// set the initial model value
					elm.select2('val', controller.$viewValue);
				});

				// Set initial value since Angular doesn't
				elm.select2('val', controller.$viewValue);
			};
		}
	};
}]);

/**********************************
 ACE EDITOR
 **********************************/
angular.module('controls').directive('ace', ['$rootScope', '$compile', function($rootScope, $compile) {
	var ACE_EDITOR_CLASS = 'ace-editor';

	// this one is called each time a token is processed in the editor. We can return some custom
	// content here which we might want to do when a drop was found :)
	var onTokenDone = function(token, content) {
		if (token.type === "drop") {
			var dropResults = $rootScope.getDropsFromText(token.value);
			if (dropResults && dropResults.length > 0) {
				var drop = dropResults[0].drop;
				if (drop) {
					return content + '<div class="drop-token-overlay"><span class="name"><i class="icon-link"></i> ' + drop._internal.name + '</span><span class="template">' + drop._internal.dropId + '</span></div>';
				}
			}
			return content + '<div class="drop-token-overlay error">Drop existiert nicht</div>';
		}
		return content;
	};

	var loadAceEditor = function(element, mode) {
		var editor = ace.edit($(element).find('.' + ACE_EDITOR_CLASS)[0], onTokenDone);
		editor.session.setMode("ace/mode/" + mode);
		editor.renderer.setShowPrintMargin(false);
		editor.$blockScrolling = Infinity;
		return editor;
	};

	return {
		restrict: 'A',
		require: '?ngModel',
		transclude: true,
		template: '<div class="transcluded" ng-transclude></div><div class="' + ACE_EDITOR_CLASS + '" ui-keypress="{\'ctrl-enter\':\'toggleFullscreen($event)\'}"></div><div class="btn btn-fullscreen trigger-tooltip" ng-click="toggleFullscreen()" data-original-title="<strong>Vollbild:</strong> Strg + Enter"><i class="icon-fullscreen"></i></div>',

		link: function(scope, element, attrs, ngModel) {
			var textarea = $(element).find('textarea');
			textarea.hide();

			var mode = attrs.ace;
			var editor = loadAceEditor(element, mode);
			var session = editor.getSession();
			scope.ace = editor;
			editor.setOption("maxLines", 20);
			editor.setOption("minLines", 4);
			session.setTabSize(2);
			session.setUseSoftTabs(false);
			window.editorsession = session;

			var detectDropClick = function(e) {
				var pos = editor.getCursorPosition();
				var token = editor.session.getTokenAt(pos.row, pos.column);
				if (token && token.type === "drop") {
					var dropGuid = token.value.replace("[drop", "").replace("]", "").trim();
					session.setAnnotations([{
						row: pos.row,
						column: pos.column,
						text: "STRG + Click, um zum Drop zu springen",
						type: "info"
					}]);
					if (e.domEvent.ctrlKey || e.domEvent.metaKey === true) {
						$rootScope.activateDrop(dropGuid);
					}
				} else {
					session.setAnnotations([]);
				}
			};

			editor.on("click", detectDropClick);

			var onChange = function (callback) {
				return function (e) {
					var newValue = session.getValue();
					if (newValue !== scope.$eval(attrs.value) && !scope.$$phase && !scope.$root.$$phase) {
						if (angular.isDefined(ngModel)) {
							scope.$apply(function() {
								ngModel.$setViewValue(newValue);
								scope.ace.resize(true);
							});
						}
					}
				};
			};

			scope.toggleFullscreen = function(e) {
				if (e) {
					e.preventDefault();
					e.stopPropagation();
				}
				$(element).toggleClass('fullscreen');

				if ($(element).hasClass('fullscreen')) {
					$('body').addClass('ace-fullscreen');
					editor.setOption("maxLines", null);
				} else {
					$('body').removeClass('ace-fullscreen');
					editor.setOption("maxLines", 20);
				}
				scope.ace.resize(true);
			};

			// Value Bind
			if (angular.isDefined(ngModel)) {
				ngModel.$formatters.push(function (value) {
					if (angular.isUndefined(value) || value === null) {
						return '';
					}
					else if (angular.isObject(value) || angular.isArray(value)) {
						throw new Error('ui-ace cannot use an object or an array as a model');
					}
					return value;
				});

				ngModel.$render = function () {
					session.setValue(ngModel.$viewValue);
				};
			}

			session.on('change', onChange());

			scope.onDropDrag = function(drop, e) {
				var editorPos = editor.renderer.screenToTextCoordinates(e.clientX, e.clientY);
				editor.moveCursorToPosition(editorPos);
			};

			// Handle Drop DropItem events
			scope.onDropDrop = function(drop) {
				editor.insert("\n" + $rootScope.getDropShortcode(drop));
			};
			
			scope.$on('$destroy', function() {
				// if we are still in fullscreen mode, remove the overflow hidden from the body
				$('body').removeClass('ace-fullscreen');
			});
		}
	};
}]);

/**********************************
 DATE PICKER
 **********************************/
angular.module('controls').directive('uiDate', [function () {
	var options;
	options = {};
	angular.extend(options, {});
	return {
		require:'?ngModel',
		link: function (scope, element, attrs, controller) {

			var getOptions = function () {
				return angular.extend({}, scope.$eval(attrs.uiDate));
			};

			var initDateWidget = function () {
				var showing = false;
				var opts = getOptions();

				// If we have a controller (i.e. ngModelController) then wire it up
				if (controller) {

					// Set the view value in a $apply block when users selects
					// (calling directive user's function too if provided)
					var _onSelect = opts.onSelect || angular.noop;
					opts.onSelect = function (value, picker) {
						scope.$apply(function() {
							showing = true;
							controller.$setViewValue(element.datepicker("getDate"));
							_onSelect(value, picker);
							element.blur();
						});
					};
					opts.beforeShow = function() {
						showing = true;
					};
					opts.onClose = function(value, picker) {
						showing = false;
					};
					element.on('blur', function() {
						if ( !showing ) {
							scope.$apply(function() {
								element.datepicker("setDate", element.datepicker("getDate"));
								controller.$setViewValue(element.datepicker("getDate"));
							});
						}
					});

					// Update the date picker when the model changes
					controller.$render = function () {
						var date = controller.$viewValue;
						if ( angular.isDefined(date) && date !== null && !angular.isDate(date) ) {
							throw new Error('ng-Model value must be a Date object - currently it is a ' + typeof date + ' - use ui-date-format to convert it from a string');
						}
						element.datepicker("setDate", date);
					};
				}

				$.datepicker.setDefaults({
					closeText: 'Schliessen', // set a close button text
					currentText: 'Öffnen', // set today text
					monthNames: ['Januar','Februar','März','April','Mai','Juni', 'Juli','August','September','Oktober','November','Dezember'], // set month names
					monthNamesShort: ['Jan','Feb','Mär','Apr','Mai','Jun','Jul','Aug','Sep','Okt','Nov','Dez'], // set short month names
					dayNames: ['Sonntag','Montag','Dienstag','Mittwoch','Donnerstag','Freitag','Samstag'],
					dayNamesShort: ['So','Mo','Di','Mi','Do','Fr','Sa'],
					dayNamesMin: ['So','Mo','Di','Mi','Do','Fr','Sa'],
					dateFormat: 'mm/dd/yy' // set format date
				});

				// If we don't destroy the old one it doesn't update properly when the config changes
				element.datepicker('destroy');
				// Create the new datepicker widget
				element.datepicker(opts);
				if ( controller ) {
					// Force a render to override whatever is in the input text box
					controller.$render();
				}
				
				// destroy the datepicker when we are going to be destroyed
				element.on('$destroy', function() {
					element.datepicker('destroy');
				});
			};
			// Watch for changes to the directives options
			scope.$watch(getOptions, initDateWidget, true);
		}
	};
}]);

/**********************************
 Date Format Helper
 **********************************/
angular.module('controls').directive('uiDateFormat', [function() {
	var directive = {
		require:'ngModel',
		link: function(scope, element, attrs, modelCtrl) {
			var dateFormat = scope.$eval(attrs.uiDateFormat) || 'mm/dd/yy';
			if ( dateFormat ) {
				// Use the datepicker with the attribute value as the dateFormat string to convert to and from a string
				modelCtrl.$formatters.push(function(value) {
					if (angular.isString(value) ) {
						return jQuery.datepicker.parseDate(dateFormat, value);
					}
					return null;
				});
				modelCtrl.$parsers.push(function(value) {
					if (value) {
						return jQuery.datepicker.formatDate(dateFormat, value);
					}
					return null;
				});
			} else {
				// Default to ISO formatting
				modelCtrl.$formatters.push(function(value) {
					if (angular.isString(value) ) {
						return new Date(value);
					}
					return null;
				});
				modelCtrl.$parsers.push(function(value) {
					if (value) {
						return value.toISOString();
					}
					return null;
				});
			}
		}
	};
	return directive;
}]);


/**********************************
 LIST
 **********************************/
angular.module('controls').directive('controlList', ['utils', 'editWindow', function(utils, editWindow) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkModel: '=',
			kkOption: '=',
			kkData: '='
		},
		template:	
			'<div class="drop-control-list">\
				<div class="btn-toolbar dock-well m-025-top">\
					<div class="btn-group">\
						<span class="btn trigger-tooltip" data-original-title="Neue {{kkOption.name || \'Elemente\'}} hinzuf&uuml;gen" ng-click="addItem(kkModel)"><i class="icon-plus"></i></span>\
					</div>\
				</div>\
				<ul class="nav nav-list well dropItem-listing" ui-sortable ng-model="kkModel">\
					<li ng-repeat="item in kkModel" ng-class="{\'active\': item==activeItem, \'item-inactive\': item._internal.visibility == \'inactive\'}">\
						<a ng-click="activateItem(item, $event)">\
							<span class="item-label">{{item._internal.name}}</span>\
							<i class="condional-active m-025-left o-05 fs-095" kk-if="item._internal.conditionalSettings.length">({{item._internal.conditionalSettings.length}} Filter aktiv)</i>\
							<span class="pull-right trigger-tooltip" ng-click="$event.stopPropagation(); toggleRemove(item, $event)" data-original-title="L&ouml;schen"><i class="icon-remove-sign toggle-popover" data-toggle="popover"></i><div class="popover"></div></span>\
							<span class="pull-right trigger-tooltip m-025-right" ng-click="copyItem(item, $event)" data-original-title="Duplizieren"><i class="icon-random"> </i> </span>\
						</a>\
					</li>\
				</ul>\
				<div class="popover fade bottom remove-popup">\
					<div class="arrow"></div>\
					<div class="popover-inner">\
						<h3 class="popover-title">L&ouml;schen</h3>\
						<div class="popover-content">\
							<small><strong>{{toRemove._internal.name}}</strong> wirklich l&ouml;schen?</small><hr/>\
							<div class="btn btn-small btn-danger" ng-click="removeItem(activeItem)">Entfernen</div>\
							<div class="btn btn-small" ng-click="toggleRemove()">Abbrechen</div>\
						</div>\
					</div>\
				</div>\
			</div>',

		/*
		{
			"name": "The lists name",
			"description": "The lists description",
			"type": "list",
			"allowPreview": true | false,
			"property": "myList",
			"itemSettings": [
				...
			],
			"itemDefaults": {
				...
			},
			"newItemName": "Neuer Slide"
		}
		*/
		link: function(scope, elem, attrs) {
			
			if (!angular.isArray(scope.kkModel)) scope.kkModel = [];

			scope.addItem = function(list) {
				var newItem = utils.Drops.addItem(list, scope.kkOption.itemDefaults, scope.kkOption.itemSettings, scope.kkData, scope.kkOption.newItemName || 'Neuer Eintrag');
				scope.activateItem(newItem);
				return newItem;
			};
			
			scope.setDefaultConfig = function(item, option) {
				if (!option) return;
				utils.Drops.setDefaultConfig(item, option.itemDefaults, option.newItemName || 'Neuer Eintrag');
				return item;
			};
			
			scope.activateItem = function(item, e) {
				if (e) {
					e.preventDefault();
				}
				editWindow.show(true, 'ListItemEditWindow', {
					item: item,
					itemParent: scope.kkData,
					itemSettings: scope.kkOption.itemSettings,
					useConditional: (scope.kkOption && typeof scope.kkOption.itemConditionals === 'boolean' ? scope.kkOption.itemConditionals : true),
					nameBinding: (scope.kkOption.itemNameBinding || null)
				});
				
				scope.activeItem = item;
			};
			
			scope.copyItem = function(item, e) {
				if (e) {
					e.preventDefault();
					e.stopPropagation();
				}
				var newItem = utils.Drops.copyItem(scope.kkModel, item, scope.kkOption.itemSettings, scope.kkData);
				scope.activateItem(newItem, e);
			};
			
			scope.toggleRemove = function(item, e) {
				var jPopup = $(elem).find('.remove-popup');	
				if (e && e.target) {
					e.preventDefault();
					e.stopPropagation();
					var jTarget = $(e.target);
					// try to position the popup to the given target
					var pos = jTarget.position();
					jPopup.css({ left: (pos.left + jTarget.width()/2 - jPopup.width()/2) + 'px', top: (pos.top + jTarget.height()/2) + 'px'});
				}
		
				if (item && e) {
					e.stopPropagation();
				}
		
				if (item) {
					scope.toRemove = item;
					jPopup.addClass('in');
				} else {
					jPopup.removeClass('in');
				}
			};

			scope.removeItem = function() {
				if (scope.toRemove && scope.kkModel.indexOf(scope.toRemove) >= 0) {
					// remove a drop
					scope.kkModel.splice(scope.kkModel.indexOf(scope.toRemove), 1);
					scope.activeItem = undefined;
				}
				$(elem).find('.remove-popup').removeClass('in');
			};			

		}
	};
}]);


/**********************************
 CONDITIONAL DIRECTIVE (Generic)
 **********************************/
angular.module('controls').directive('conditional', ['utils', '$timeout', 'core', '$sce', function(utils, $timeout, core, $sce) {
	return {
		restrict: 'E',
		replace: true,
		transclude: true,
		
		scope: {
			kkSettings: '='	// binding to the conditional settings
		},

		templateUrl: core.adminURL + 'js/controls/conditional.html.js?' + new Date().getTime(),

		link: function(scope, elem, attrs, ctrl, transclude) {
			var conditional = window.conditional;
			scope.conditionals = conditional;

			// bind the transcluded content to the emptyDesc scope property
			transclude(scope, function(clone, scope) {
				if (clone && clone.length) { 
					scope.emptyDesc = $sce.trustAsHtml($('<div>').append(clone).html());
				}
				else scope.emptyDesc = $sce.trustAsHtml('<div>Dieses Drop wird momentan auf jeder Seite angezeigt. Füge Filter hinzu, wenn du möchtest dass dieses Drop nur unter gewissen Umständen angezeigt wird.<br/>Weitere Informationen zu den Darstellungsfiltern findest du in unserem <a target="_blank" href="https://kreativkonzentrat.de/Wiki?page=dropper/details/einfuegemethoden#cmsanchor-darstellungsfilter">Wiki</a>.</div>');
			});

			scope.operands = {
				and: 'und',
				or: 'oder'
			};
			
			scope.filterSections = {
				"Global": { label: "Global" },
				"Listing": { label: "Artikellisting" },
				"Article": { label: "Artikeldetail" },
				"Time": { label: "Zeit" },
				"CMS": { label: "Inhalte" },
				"Session": { label: "Session" },
				"Others": { label: "Weitere" }
			};
			
			// show all conditionals that are already there
			$timeout(function() {
				elem.find('.conditional').addClass('conditional-visible noanim');
			});
			
			/**
			 * Gets an object with key = conditional key with filters that are
			 * matching the given section
			 */
			var getFiltersForSection = function(section) {
				var filters = {};
				for(var condKey in conditional) {
					var cond = conditional[condKey];
			
					if (cond && cond.legacy) continue;
			
					if (section.toLowerCase() === 'others' && (!cond.section || cond.section.toLowerCase() === 'others')) {
						filters[condKey] = cond;
					}
					if (cond.section && section.toLowerCase() === cond.section.toLowerCase()) {
						filters[condKey] = cond;
					}
				}
				return filters;
			};
			
			// enhance the filter sections with the filters
			for(var section in scope.filterSections) {
				var filters = getFiltersForSection(section);
				if (!filters || !Object.keys(filters).length) { 
					delete scope.filterSections[section];
				} else {
					scope.filterSections[section].filters = filters;
				}
			}
			
			/**
			 * Adds a new Conditional to the currently active Drop Conditionals
			 * @ckey - the Conditional Key of the Conditional to add
			 */
			scope.addConditional = function(ckey) {
				if (!scope.kkSettings) {
					scope.kkSettings = [];
				}

				scope.kkSettings.push({ 
					setting: {
						operand: 'and', 
						choices: {}
					}, 
					conditional: ckey, 
					equality: true
				});
				
				// wait for the ui to update
				$timeout(function() {
					var container = elem.closest('.modal-body').eq(0);
					var cond = elem.find('.conditional:last');
				
					// show conditional
					cond.addClass('conditional-visible');
	
					$timeout(function() {
						cond.addClass('noanim');
					}, 800);
	
					if (!cond.length || !container.length) return;
					
					$timeout(function() {
						var condElem = cond.get(0);
						if (condElem && typeof condElem.scrollIntoView === 'function') {
							condElem.scrollIntoView({behavior: "smooth", block: "start"});
						}
					}, 10);
				});
			};
			
			/**
			 * Removes a Conditional from the active Drop
			 */
			scope.removeConditional = function(condSetting) {
				scope.kkSettings.splice(scope.kkSettings.indexOf(condSetting), 1);
			};
			
			/**
			 * Toggles the equality option of a conditional
			 */
			scope.toggleConditionalEquality = function(cond) {
				if (!cond) return;
				if (typeof(cond.equality) === 'undefined' || cond.equality === true) { 
					cond.equality = false; // when nothing is set yet, it has been evaled to "true" before, therefore we return false
				} else { 
					cond.equality = true;
				}
			};

			/**
			 * Generates the Equality label
			 */
			scope.conditionEqualityLabel = function(condSetting) {
				if (!condSetting) return "";
				if (condSetting.equality === false) return "ist nicht";
				return "ist";
			};
		}
	};
}]);

/**********************************
 CONDITIONAL CONTROL
 **********************************/
angular.module('controls').directive('controlConditional', ['utils', '$timeout', 'core', 'modal', function(utils, $timeout, core, modal) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkOption: '=',
			kkModel: '=',	// an object that has following properties: conditional, conditionalSettings
		},
		template: '\
			<div class="btn" ng-click="showModal()" style="white-space: nowrap">\
				<i class="icon-filter"></i>\
				<small kk-if="!kkModel.conditionalSettings.length">Konditionen w&auml;hlen</small>\
				<small kk-if="kkModel.conditionalSettings.length">\
					<span ng-repeat="setting in kkModel.conditionalSettings">\
						<strong ng-if="setting.equality">{{conditionals[setting.conditional].name}}</strong><s ng-if="!setting.equality"><strong>{{conditionals[setting.conditional].name}}</strong></s>\
						<span ng-if="$index < kkModel.conditionalSettings.length-1"> {{translateOperand(setting.setting.operand)}} </span>\
					</span>\
				</small>\
				<i class="icon-remove-sign fadebtn" style="margin-left: 5px; margin-right: -5px; vertical-align:middle;" title="Konditionen entfernen" ng-click="reset($event)" kk-if="kkModel.conditionalSettings.length"></i>\
			</div>',
		link: function(scope, elem, attrs) {
			if (!angular.isObject(scope.kkModel)) {
				scope.kkModel = {
					conditionalSettings: [],
					condition: "",
					_conditional: true
				};
			}
			
			if (!angular.isArray(scope.kkModel.conditionalSettings)) scope.kkModel.conditionalSettings = [];
			if (!angular.isString(scope.kkModel.condition)) scope.kkModel.condition = "";
			scope.kkModel['_conditional'] = true;
			scope.conditionals = window.conditional;
			
			scope.buttonLabel = "";
			
			/**
			 * reset condition
			 */
			scope.reset = function(e) {
				if (e) e.stopPropagation();

				scope.kkModel = {
					conditionalSettings: [],
					condition: "",
					_conditional: true
				};
			};
			
			/**
			 * Show condition modal
			 */
			scope.showModal = function() {
				modal.show(
					'ControlConditionalModal', 
					// close callback
					function (res) {
						if (angular.isObject(res)) {
							scope.kkModel.condition = res.condition;
							scope.kkModel.conditionalSettings = res.conditionalSettings;
						}
					},
					// injects
					{
						conditionalSettings: scope.kkModel.conditionalSettings
					}
				);
			};
			
			/**
			 * Translates operand names
			 */
			scope.translateOperand = function(op) {
				switch(op) {
					case 'and':
						return 'und';
					case 'or':
						return 'oder';
					default:
						return '';
				}	
				return '';
			};
		}
	};
}]);

/**********************************
 SELECTOR INPUT
 **********************************/
angular.module('controls').directive('controlSelectorInput', ['core', '$rootScope', '$timeout', function(core, $rootScope, $timeout) {
	return {
		restrict: 'E',
		replace: true,
		scope: {
			kkModel: '=',
			kkOption: '=',
			kkData: '='
		},
		template: '\
		<div class="selector-input-control" ng-class="{\'selector-input-control--focused\': editMode}">\
			<div class="selector-input-control-left">\
				<input type="text" class="selector-input-control--input" placeholder="CSS- oder Zonenselektor eintragen" spellcheck="false" ng-model="kkModel" ng-show="editMode || !kkModel"/>\
				<div class="selector-input-control--selectors" ng-class="{\'o-07\': !kkData.replaceMethod}" ng-click="setEditMode(true)" ng-show="!editMode && kkModel">\
					<div class="selector-input-control--selector" title="{{selectorInfo.info}}" data-selector-type="{{selectorInfo.type}}" ng-repeat="selectorInfo in selectorParts" ng-click="editPart(selectorInfo, $event)">{{selectorInfo.name}} <span class="selector-input-control--selector-remove" ng-click="removePartial(selectorInfo, $event)">&times</span></div>\
				</div>\
			</div>\
			<div class="selector-input-control-right trigger-tooltip" data-original-title="Position suchen" ng-click="setPickerVisible()"><i class="icon-screenshot"></i></div>\
		</div>',

		link: function(scope, elem, attrs) {

			scope.selectorParts = [];
			scope.editMode = false;
			
			/**
			 * show selectors when we blur
			 */
			var $input = elem.find('.selector-input-control--input');
			$input.blur(function() {
				scope.$apply(function() {
					scope.cleanupModel();
					scope.setEditMode(false);
				});
			});

			// activate edit mode no input focus
			$input.focus(function() {
				scope.$apply(function() {
					scope.editMode = true;
				});
			});
			
			// leave edit mode on enter
			$input.keydown(function(e) {
				if (e.keyCode === 13) {
					scope.$apply(function() {
						scope.setEditMode(false);
					});
				}
			});

			/**
			 * Show the picker dialog from the insert conditions modal.
			 * HACK: We bubble up the modal parent chain until we find the correct
			 * funtion to trigger
			 */
			scope.setPickerVisible = function() {
				var tmpScope = scope;

				while (tmpScope.$parent) {
					if (typeof tmpScope.$parent.setPickerVisible === 'function') {
						tmpScope.$parent.setPickerVisible(true);
						break;
					}
					tmpScope = tmpScope.$parent;
				}
			};

			/**
			 * sets the editing mode of this control
			 */
			scope.setEditMode = function(active) {
				scope.editMode = active;
				
				if (active) {
					$timeout(function() {
						$input.focus();
						$input.get(0).setSelectionRange(scope.kkModel.length, scope.kkModel.length);
					});
				}
			};

			/**
			 * edit a selector part, this will go to edit mode
			 * and select the selector inside the text
			 */
			scope.editPart = function(selectorInfo, e) {

				var getMatchBounds = function(longText, part) {
					
					var parts = longText.split(',');
					var length = 0;
					
					for (var i=0; i<parts.length; i++) {
						var piece = parts[i];
						if (piece.trim() === part.trim()) {
							return {
								start: length + piece.indexOf(part),
								end: length + piece.indexOf(part) + part.length
							};
						}
						
						length += piece.length + 1;
					}

					return false;
				};
				
				e.stopPropagation();
				e.preventDefault();
				scope.editMode = true;
				
				$timeout(function() {
					var input = $input.get(0);
					$input.focus();
					var text = scope.kkModel;
					// select the text
					var bounds = getMatchBounds(text, selectorInfo.selector);
					if (bounds) {
						input.setSelectionRange(bounds.start, bounds.end);
					}
				});
			};

			/**
			 * removes a partial selector
			 */
			scope.removePartial = function(info, e) {

				e.stopPropagation();
				e.preventDefault();

				var idx = scope.selectorParts.indexOf(info);
				if (idx >= 0) {
					scope.selectorParts.splice(idx, 1);
					var cleaned = '';
					for (var i=0; i<scope.selectorParts.length; i++) {
						cleaned += scope.selectorParts[i].selector;
						if (i < scope.selectorParts.length-1) cleaned += ', ';
					}
					scope.kkModel = cleaned;
				}
			};

			/**
			 * tries to get a visual representation of the selector parts
			 */
			scope.getSelectors = function() {

				if (!scope.kkModel) {
					return [];
				}

				var results = [];
				var parts = scope.kkModel.split(',');

				for (var i=0; i<parts.length; i++) {
					var part = parts[i].trim();
					if (part) {
						var res = scope.getZoneInfo(part);
						if (res) results.push(scope.getZoneInfo(part));
					}
				}
				return results;
			};


			/**
			 * gets extended informations about the given selector (part)
			 */
			scope.getZoneInfo = function(selector) {

				// normal selector?
				if (selector.indexOf('@') !== 0) {
					return {
						name: selector,
						info: 'CSS Selektor',
						selector: selector,
						type: (selector.split(' ').reverse()[0].indexOf('#') === 0 ? 'id' : 'class')
					};
				}

				// is this a zone wrapped arround a drop?
				if (selector.indexOf('@drop.') === 0) {
					var guid = selector.substr(6);
					var drop = $rootScope.getDropByGuid(guid);
					if (drop && drop._internal && drop._internal.name) {
						return {
							name: drop._internal.name,
							info: "Drop",
							selector: selector,
							type: 'drop'
						};
					}
				}
				
				// is this a zone from a template integration?
				if (selector.indexOf('@') === 0) {
					var zoneId = selector.substr(1);
					var idxMatches = zoneId.match(/\(\d+\)/g, zoneId);
					zoneId = zoneId.replace(/\(\d+\)/g, '($index)');

					if (core.templateZones && core.templateZones[zoneId]) {
						// get the definition
						var zoneDef = core.templateZones[zoneId];

						// replace indexes
						var name = zoneDef.namedPath.join(' » ');

						if (idxMatches && idxMatches.length) {
							var matchIdx = 0;
							while (matchIdx < idxMatches.length) {
								var namedIdx =  parseInt(idxMatches[matchIdx].replace('(', '').replace(')', ''), 10) + 1;
								name = name.replace('$index', namedIdx);
								matchIdx++;
							}
						}
						
						return {
							name: name,
							info: "Template Zone",
							selector: selector,
							type: 'zone'
						};
					}
				}

				// default
				return {
					name: selector,
					selector: selector,
					info: "Unbekannte Zone",
					type: 'zone'
				};
			};
			
			/**
			 * cleans up the selector
			 */
			scope.cleanupModel = function() {

				if (!scope.kkModel) return;

				// cleanup
				var cleaned = '';
				for (var i=0; i<scope.selectorParts.length; i++) {
					cleaned += scope.selectorParts[i].selector;
					if (i < scope.selectorParts.length-1) cleaned += ', ';
				}
				if (scope.kkModel !== cleaned) scope.kkModel = cleaned;
			};
			
			scope.$watch('kkModel', function() {

				scope.selectorParts.splice(0, scope.selectorParts.length);

				// add new selectors
				scope.selectorParts.push.apply(scope.selectorParts, scope.getSelectors());	
			});
		}
	};
}]);