angular.module('app').directive('kkNotifications', ['message', '$timeout', function(message, $timeout) {

	return {

		restrict: 'E',
		replace: true,
		scope: {},

		template: '\
		<div ng-hide="basic" class="dropper-notifications pull-left btn-group" ng-class="{ show: state.open || hasNotifications() }">\
			<span class="dropper-notifications-button btn no-bg btn-large trigger-tooltip" ng-class="{ active: state.open }" ng-click="$event.stopPropagation(); state.open = !state.open;" data-original-title="Nachrichtenzentrale">\
				<small><i class="icon-bell"></i>&#xfeff;</small>\
				<div class="dropper-notifications-button-badge" data-notification-type="{{state.dominantType}}"></div>\
			</span>\
			<div class="dropper-notification-dropdown dropdown-menu" ng-class="{ open: state.open }" ng-click="$event.stopPropagation()">\
				<div class="arrow"></div>\
				<div class="dropper-notification-dropdown-header relative p-1 bbottom" ng-if="hasNotifications()">\
					<h4 class="m-0 d-inline-block">\
						{{messageService.messages.length}} Nachrichten\
					</h4>\
					<span class="close" ng-click="state.open=false">&times;</span>\
				</div>\
				<h5 ng-if="!hasNotifications()" class="p-15-left p-15-right p-05-top p-05-bottom tcenter">\
					Keine neuen Nachrichten\
				</h5>\
				<div class="dropper-notifications-wrapper" ng-if="hasNotifications()">\
					<div ng-repeat="(type, group) in notificationGroups" ng-if="groupTypeVisible(type)" class="dropper-notification-type-group" data-notification-type="{{type}}">\
						<div class="dropper-notification-group" ng-repeat="(title, notifications) in group">\
							<div class="dropper-notification-group-header">\
								<div class="dropper-notification-group-title" data-notification-type="{{type}}" ng-bind-html-unsafe="title"></div>\
								<div class="close" title="Alle schliessen" ng-click="dismissNotificationGroup(notifications)">&times;</div>\
							</div>\
							<div class="dropper-notifications-group-wrapper" ng-if="hasNotificationBodies(notifications)" data-notification-type="{{notification.type}}">\
								<div class="dropper-notification" ng-repeat="notification in notifications" ng-if="notification.body">\
									<div class="dropper-notification-body" ng-if="notification.allowHTML" ng-bind-html-unsafe="notification.body"></div>\
									<div class="dropper-notification-body" ng-if="!notification.allowHTML">{{notification.body}}</div>\
									<a class="close" ng-if="notifications.length > 1" ng-click="dismissNotification(notification)">&times;</a>\
								</div>\
							</div>\
						</div>\
					</div>\
				</div>\
			</div>\
		</div>',

		link: function(scope, elem, attrs) {

			// basic notifications?
			scope.basic = attrs.basic === "true" ? true : false;

			// reference dropdown and button elements
			var $dropdown = elem.find('.dropper-notification-dropdown');
			var $btn = elem.find('.dropper-notifications-button');

			// bind the messageService to the scope (so that we can $watch it)
			scope.messageService = message;

			// count of messages when the last change happened on the message service
			var oldNotificationCount = 0;

			// init state
			if (!scope.state) scope.state = {
				open: false,			// whether this should be shown or hidden (two way)
				dominantType: 'none'	// most severe message type, that has messages active
			};

			// we will group messages by type and title
			scope.notificationGroups = {
				error: {},
				warning: {},
				success: {},
				info: {}
			};


			// dismisses a single message
			scope.dismissNotification = function(msg) {
				message.removeMessage(msg);
			};


			// dismisses all messages from a group			
			scope.dismissNotificationGroup = function(notifications) {
				for (var i=0; i<notifications.length; i++) {
					message.removeMessage(notifications[i]);
				}	
			};

			// check if we've got messages to display
			scope.hasNotifications = function() {
				return message.messages && message.messages.length ? true : false;
			};

			// checks if minimum one notification has a notification body set
			scope.hasNotificationBodies = function(notifications) {
				for (var i=0; i<notifications.length; i++) {
					if (notifications[i].body && notifications[i].body.trim()) return true;
				}
				
				return false;
			};

			// checks if a given group type (info, error, warning, success) has notifications to it			
			scope.groupTypeVisible = function(type) {
				return scope.notificationGroups[type] && Object.keys(scope.notificationGroups[type]).length;
			};
			
			// watch for changes in the messages array
			scope.$watch('messageService.messages', function(messages) {
				
				var notificationGroups = scope.notificationGroups;
				
				// reset message groups
				notificationGroups.error = {};
				notificationGroups.warning = {};
				notificationGroups.success = {};
				notificationGroups.info = {};

				// rebuild message groups
				for (var i=0; i<messages.length; i++) {

					var msg = messages[i];
					if (!notificationGroups[msg.type][msg.title]) {
						notificationGroups[msg.type][msg.title] = [];
					}
					// prepend message to the array, so that most recent ones show up first
					notificationGroups[msg.type][msg.title].unshift(msg);
				}

				// update state
				if (Object.keys(notificationGroups.error).length) {
					scope.state.dominantType = 'error';
				} 
				else if (Object.keys(notificationGroups.warning).length) {
					scope.state.dominantType = 'warning';
				}
				else if (Object.keys(notificationGroups.success).length) {
					scope.state.dominantType = 'success';
				}
				else if (Object.keys(notificationGroups.info).length) {
					scope.state.dominantType = 'info';
				}
				else {
					scope.state.dominantType = 'none';
				}
				
				// when message count got bigger we automaticly show up
				if (messages.length > oldNotificationCount) scope.state.open = true;
				oldNotificationCount = messages.length;

				// close when no messages are left
				if (!messages.length) scope.state.open = false;

			}, true);
			
			// recalculates the correct position for the dropdown
			var oldDropdownPosition = {left: null, top: null};
			var positionDropdown = function() {

				// attach the dropdown to #app container
				if (!$dropdown.parent().is('#app')) {
					$dropdown.appendTo('#app');	
				}
				
				// do nothing in basic mode
				if (scope.basic) return;

				var pos = elem.offset();
				var dropdownPos = {
					top: Math.round(pos.top + elem.outerHeight(false) - $(document).scrollTop()) + 'px',
					left: Math.round(pos.left)
				};
	
				// no change?
				if (oldDropdownPosition.left === dropdownPos.left && oldDropdownPosition.top === dropdownPos.top) return;

				// attach the popup to another container, so that it is topmost
				$dropdown.css(dropdownPos);
				oldDropdownPosition = dropdownPos;
				
			};

			
			// position dropdown after open
			scope.$watch('state.open', function(isOpen) {

				$timeout(function() {
					positionDropdown();
				});

			});

			// set dropdown position
			positionDropdown();

			// since we have detached the dropdown we need to
			// manually remove it once the scope is being destroyed
			scope.$on('$destroy', function() {
				$dropdown.remove();
			});

		}
		
	};
}]);


angular.module('app').directive('ngBindHtmlUnsafe', ['$sce', function($sce){
	return {
		scope: {
			ngBindHtmlUnsafe: '=',
		},
		template: "<div ng-bind-html='trustedHtml'></div>",
		link: function($scope, iElm, iAttrs, controller) {
			$scope.updateView = function() {
				$scope.trustedHtml = $sce.trustAsHtml($scope.ngBindHtmlUnsafe);
			};

			$scope.$watch('ngBindHtmlUnsafe', function(newVal, oldVal) {
				$scope.updateView(newVal);
			});
		}
	};
}]);



angular.module('app').directive('contenteditable', [function() {
	return {
		restrict: 'A', // only activate on element attribute
		require: '?ngModel', // get a hold of NgModelController
		link: function(scope, element, attrs, ngModel) {
			
			if (!ngModel) return; // do nothing if no ng-model
			
			var defaultsTo = attrs.defaultsTo ? scope.$eval(attrs.defaultsTo) : 'Unbenannt';

			// Specify how UI should be updated
			ngModel.$render = function() {
				element.text(ngModel.$viewValue || '');
			};
			
			// Listen for change events to enable binding
			element.on('blur keyup change', function() {
				scope.$evalAsync(read);
			});

			element.on('blur', function() {
				var text = element.text().trim();
				if (text === "") { 
					element.text(defaultsTo);
					ngModel.$setViewValue(defaultsTo);
				}
			});

			// Write data to the model
			function read() {

				// html inside? transform it to text!
				element.find('*').each(function() {
					var $inner = $(this);
					$inner.replaceWith($inner.text());
				});
				
				// remove all remaining elements
				element.find('*').remove();
				
				// add a br so that firefox has no bugs :(
				element.append('<br>');

				// get the final text content from our element
				var text = element.text();

				// update the model value
				ngModel.$setViewValue(text);

				// add empty class when text is empty
				text === "" ? element.addClass('empty') : element.removeClass('empty');
			}
		}
	};
}]);

angular.module('app').directive('kkMouseleave', ['$parse', function($parse) {
	return {
		restrict: 'A', // only activate on element attribute
		link: function(scope, element, attrs) {
			element.bind('mouseleave', function(){
				var fn = $parse(attrs.kkMouseleave);
				scope.$apply(function(){
					fn(scope, {});
				});
			});
		}
	};
}]);

/**
 * Bind one or more handlers to particular keys or their combination
 * @param hash {mixed} keyBindings Can be an object or string where keybinding expression of keys or keys combinations and AngularJS Exspressions are set. Object syntax: "{ keys1: expression1 [, keys2: expression2 [ , ... ]]}". String syntax: ""expression1 on keys1 [ and expression2 on keys2 [ and ... ]]"". Expression is an AngularJS Expression, and key(s) are dash-separated combinations of keys and modifiers (one or many, if any. Order does not matter). Supported modifiers are 'ctrl', 'shift', 'alt' and key can be used either via its keyCode (13 for Return) or name. Named keys are 'backspace', 'tab', 'enter', 'esc', 'space', 'pageup', 'pagedown', 'end', 'home', 'left', 'up', 'right', 'down', 'insert', 'delete'.
 * @example <input ui-keypress="{enter:'x = 1', 'ctrl-shift-space':'foo()', 'shift-13':'bar()'}" /> <input ui-keypress="foo = 2 on ctrl-13 and bar('hello') on shift-esc" />
 **/
angular.module('app').directive('uiKeypress', ['$parse', function ($parse) {
	return {
		link: function (scope, elm, attrs) {
			var keysByCode = {
				8: 'backspace',
				9: 'tab',
				13: 'enter',
				27: 'esc',
				32: 'space',
				33: 'pageup',
				34: 'pagedown',
				35: 'end',
				36: 'home',
				37: 'left',
				38: 'up',
				39: 'right',
				40: 'down',
				45: 'insert',
				46: 'delete'
			};

			var params, paramsParsed, expression, keys, combinations = [];
			try {
				params = scope.$eval(attrs.uiKeypress);
				paramsParsed = true;
			} catch (error) {
				params = attrs.uiKeypress.split(/\s+and\s+/i);
				paramsParsed = false;
			}

			// Prepare combinations for simple checking
			angular.forEach(params, function (v, k) {
				var combination = {};
				if (paramsParsed) {
					// An object passed
					combination.expression = $parse(v);
					combination.keys = k;
				} else {
					// A string passed
					v = v.split(/\s+on\s+/i);
					combination.expression = $parse(v[0]);
					combination.keys = v[1];
				}

				keys = {};
				angular.forEach(combination.keys.split('-'), function (value) {
					keys[value] = true;
				});
				combination.keys = keys;
				combinations.push(combination);
			});

			// Check only mathcing of pressed keys one of the conditions
			elm.bind('keydown', function (event) {
				// No need to do that inside the cycle
				var altPressed = event.metaKey || event.altKey;
				var ctrlPressed = event.ctrlKey;
				var shiftPressed = event.shiftKey;

				// Iterate over prepared combinations
				angular.forEach(combinations, function (combination) {

					var mainKeyPressed = (combination.keys[keysByCode[event.keyCode]] || combination.keys[event.keyCode.toString()]) || false;

					var altRequired = combination.keys.alt || false;
					var ctrlRequired = combination.keys.ctrl || false;
					var shiftRequired = combination.keys.shift || false;

					if (mainKeyPressed &&
						( altRequired == altPressed   ) &&
						( ctrlRequired == ctrlPressed  ) &&
						( shiftRequired == shiftPressed )
						) {
						// Run the function
						scope.$apply(function () {
							combination.expression(scope, { '$event': event });
						});
					}
				});
			});
		}
	};
}]);

/*
 * Defines the ui-if tag. This removes/adds an element from the dom depending on a condition
 * Originally created by @tigbro, for the @jquery-mobile-angular-adapter
 * https://github.com/tigbro/jquery-mobile-angular-adapter
 */
angular.module('app').directive('kkIf', [function () {
	return {
		transclude: 'element',
		priority: 1000,
		terminal: true,
		restrict: 'A',
		compile: function (element, attr, linker) {
			return function (scope, iterStartElement, attr) {
				iterStartElement[0].doNotMove = true;
				var expression = attr.kkIf;
				var lastElement;
				var lastScope;
				scope.$watch(expression, function (newValue) {
					if (lastElement) {
						lastElement.remove();
						lastElement = null;
					}
					if (lastScope) {
						lastScope.$destroy();
						lastScope = null;
					}
					if (newValue) {
						lastScope = scope.$new();
						linker(lastScope, function (clone) {
							lastElement = clone;
							iterStartElement.after(clone);
						});
					}
					// Note: need to be parent() as jquery cannot trigger events on comments
					// (angular creates a comment node when using transclusion, as ng-repeat does).
					iterStartElement.parent().trigger("$childrenChanged");
				});
			};
		}
	};
}]);

angular.module('app').directive('animate', [function(){
	return function(scope, elm, attrs) {
		$(elm).css({  opacity: 0,
			'-webkit-transform': 'scale(0)',
			'-moz-transform': 'scale(0)',
			'-o-transform': 'scale(0)',
			'transform': 'scale(0)',
			'margin-left': -1*$(elm).outerWidth() + 'px'
		});

		setTimeout(function(){
			$(elm).css({  opacity: 1,
				'-webkit-transform': '',
				'-moz-transform': '',
				'-o-transform': '',
				'transform': '',
				'margin-left': '',
				'transition': 'all 300ms ease-out',
				'-webkit-transition': 'all 300ms ease-out',
				'-moz-transition': 'all 300ms ease-out',
				'-o-transition': 'all 300ms ease-out'
			});
		}, 100);
	};
}]);


angular.module('app').directive("kkWindow", [function() {
	return {
		restrict: 'A',
		link: function(scope, elem, attrs) {
			$(elem).addClass('kk-window');

			attrs.$observe('kkWindow', function(value) {
				if (value == 'true') {
					$(elem).addClass('open');
					$(elem).removeClass('inactive');
					$(elem).parents('.kk-window').addClass('inactive');
				} else {
					$(elem).removeClass('open');
					$($(elem).parents('.kk-window.open.inactive')[0]).removeClass('inactive');
				}
			});
		}
	};
}]);

angular.module('app').directive('uiSortable', ['$timeout', '$log', function ($timeout, $log) {
	return {
		require: '?ngModel',
		link: function (scope, element, attrs, ngModel) {
			var savedNodes;

			function combineCallbacks(first, second) {
				if(second && (typeof second === "function")) {
					return function (e, ui) {
						first(e, ui);
						second(e, ui);
					};
				}
				return first;
			}

			var opts = {};

			var callbacks = {
				receive: null,
				remove: null,
				start: null,
				stop: null,
				update: null
			};

			angular.extend(opts, { distance: 10 });

			if(ngModel) {

				// When we add or remove elements, we need the sortable to 'refresh'
				// so it can find the new/removed elements.
				scope.$watch(attrs.ngModel + '.length', function () {
					// Timeout to let ng-repeat modify the DOM
					$timeout(function () {
						element.sortable("refresh");
					});
				});

				callbacks.start = function (e, ui) {
					// Save the starting position of dragged item
					ui.item.sortable = {
						index: ui.item.index(),
						cancel: function () {
							ui.item.sortable._isCanceled = true;
						},
						isCanceled: function () {
							return ui.item.sortable._isCanceled;
						},
						_isCanceled: false
					};
				};

				callbacks.activate = function (e, ui) {
					// We need to make a copy of the current element's contents so
					// we can restore it after sortable has messed it up.
					// This is inside activate (instead of start) in order to save
					// both lists when dragging between connected lists.
					savedNodes = element.contents();

					// If this list has a placeholder (the connected lists won't),
					// don't inlcude it in saved nodes.
					var placeholder = element.sortable('option', 'placeholder');

					// placeholder.element will be a function if the placeholder, has
					// been created (placeholder will be an object).  If it hasn't
					// been created, either placeholder will be false if no
					// placeholder class was given or placeholder.element will be
					// undefined if a class was given (placeholder will be a string)
					if(placeholder && placeholder.element && typeof placeholder.element === 'function') {
						// exact match with the placeholder's class attribute to handle
						// the case that multiple connected sortables exist and
						// the placehoilder option equals the class of sortable items
						var excludes = element.find('[class="' + placeholder.element()
							.attr('class') + '"]');

						savedNodes = savedNodes.not(excludes);
					}
				};

				callbacks.update = function (e, ui) {
					// Save current dropItem position but only if this is not a second
					// update that happens when moving between lists because then
					// the value will be overwritten with the old value
					if(!ui.item.sortable.received) {
						ui.item.sortable.dropindex = ui.item.index();

						// Cancel the sort (let ng-repeat do the sort for us)
						// Don't cancel if this is the received list because it has
						// already been canceled in the other list, and trying to cancel
						// here will mess up the DOM.
						element.sortable('cancel');
					}

					// Put the nodes back exactly the way they started (this is very
					// important because ng-repeat uses comment elements to delineate
					// the start and stop of repeat sections and sortable doesn't
					// respect their order (even if we cancel, the order of the
					// comments are still messed up).
					savedNodes.detach()
						.appendTo(element);

					// If received is true (an item was dropped in from another list)
					// then we add the new item to this list otherwise wait until the
					// stop event where we will know if it was a sort or item was
					// moved here from another list
					if(ui.item.sortable.received && !ui.item.sortable.isCanceled()) {
						scope.$apply(function () {
							ngModel.$modelValue.splice(ui.item.sortable.dropindex, 0,
								ui.item.sortable.moved);
						});
					}
				};

				callbacks.stop = function (e, ui) {
					// If the received flag hasn't be set on the item, this is a
					// normal sort, if dropindex is set, the item was moved, so move
					// the items in the list.
					if(!ui.item.sortable.received &&
						('dropindex' in ui.item.sortable) && !ui.item.sortable.isCanceled()) {

						scope.$apply(function () {
							ngModel.$modelValue.splice(
								ui.item.sortable.dropindex, 0,
								ngModel.$modelValue.splice(ui.item.sortable.index, 1)[0]);
						});
					}
				};

				callbacks.receive = function (e, ui) {
					// An item was dropped here from another list, set a flag on the
					// item.
					ui.item.sortable.received = true;
				};

				callbacks.remove = function (e, ui) {
					// Remove the item from this list's model and copy data into item,
					// so the next list can retrive it
					if(!ui.item.sortable.isCanceled()) {
						scope.$apply(function () {
							ui.item.sortable.moved = ngModel.$modelValue.splice(
								ui.item.sortable.index, 1)[0];
						});
					}
				};

				scope.$watch(attrs.uiSortable, function (newVal, oldVal) {
					angular.forEach(newVal, function (value, key) {
						if(callbacks[key]) {
							if(key === 'stop') {
								// call apply after stop
								value = combineCallbacks(
									value, function () {
										scope.$apply();
									});
							}
							// wrap the callback
							value = combineCallbacks(callbacks[key], value);
						}
						element.sortable('option', key, value);
					});
				}, true);

				angular.forEach(callbacks, function (value, key) {
					opts[key] = combineCallbacks(value, opts[key]);
				});

			} else {
				$log.info('ui.sortable: ngModel not provided!', element);
			}

			// Create sortable
			element.sortable(opts);
		}
	};
}]);

/**
 * Makes a Drop Draggable!
 * The Drop can then be thrown on elemets that have the attribute "kk-drop-droppable" set
 */
angular.module('app').directive('kkDropDraggable', [function() {
	return {
		restrict: 'A',
		link: function(scope, elm, attrs) {
			var drop = scope.$eval(attrs.kkDrop);
			var options = scope.$eval(attrs.kkDropDraggable);

			if(!drop){
				drop = {
					_isNewPlaceholder : true,
					"_internal" : {
						name: "Neues Drop"
					}
				};
			}

			var defaultOptions = {
				revert: 'invalid',
				distance: 10,
				cursor: 'move',
				appendTo: 'body',
				helper: function() {
					var fixOffsetBug = 'style="margin-top: -' + $(document).scrollTop() + 'px"'; // there is an offset bug when the body is scrolled and this is dragged from a fixed pane. this is the fix for it
					return $('<div class="drop-drag-item" ' + fixOffsetBug + '><small>' + (drop._isNewPlaceholder ? 'Drop erstellen und verlinken' : 'Drop verlinken') + '</small><br/><span><i class="icon-link"></i> ' + drop._internal.name + '</span></div>');
				}
			};
			options = $.extend({}, defaultOptions, options);
			elm.data('drop', drop);
			elm.addClass('kk-drop-draggable');
			elm.draggable(options);
		}
	};
}]);

/**
 * Drop Target for draggable Drops
 * If the scope of another Control has a method "onDropDrop" it will be called
 */
angular.module('app').directive('kkDropDroppable', ['$rootScope', function($rootScope) {
	return {
		restrict: 'A',
		link: function(scope, elm, attrs) {
			var options = scope.$eval(attrs.kkDropDroppable);

			var callback = scope.$eval(attrs.kkDropDropped);
			var callbackData = scope.$eval(attrs.kkDropDroppedData);
			
			var isDragging = false;
			var isMouseOver = false;

			// lets perform a mouseover detection manually because otherwise
			// this droppable can be used even if there is another element overlapping it
			elm.mouseenter(function(){
				isMouseOver = true;
			});
			elm.mouseleave(function(){
				isMouseOver = false;
			});

			// generate the droppable settings
			var defaultOptions = {
				accept: '.drop-drag-item,.kk-drop-draggable',
				activeClass: 'drop-droppable-active',
				hoverClass: 'drop-droppable-hover',
				drop: function(e, ui){

					if(!isMouseOver) {
						console.log('mouse it not over me, most likely because i am covered by something, aborting drop', elm);
						return;
					}

					var drop = $(ui.draggable).data('drop');

					// is this the new placeholder drop? if so, ask which drop we want to have before inserting
					if(drop && drop._isNewPlaceholder){
						$rootScope.getNewDropUserSelect(function(newDrop){
							// callback
							if(newDrop && (typeof(scope.onDropDrop) === 'function' || typeof(callback) === 'function')){
								// set no replace method and selector for this one, as we want to link it only
								newDrop._internal.replaceMethod = "";
								newDrop._internal.selector = "";
								// get the groupname of the active drop, so that we can put it into the same group
								if($rootScope.activeDrop){
									var groupName = $rootScope.groupLabel($rootScope.activeDrop._internal.name);
									if(groupName !== $rootScope.defaultGroup){
										newDrop._internal.name = groupName + ": " + newDrop._internal.name;
									}
								}
								setTimeout(function(){
									if(typeof(scope.onDropDrop) === 'function'){
										scope.onDropDrop(newDrop, e);
									}
									if(typeof(callback) === 'function'){
										callback(newDrop, callbackData);
									}
								});
							}
						}, false);
					}
					// this is a regular drop... call the callback if possible
					else{
						// callback
						if(drop && typeof(scope.onDropDrop) === 'function'){
							scope.onDropDrop(drop, e);
						}
						if(drop && typeof(callback) === 'function'){
							callback(drop, callbackData);
						}
					}
				},
				over: function(e, ui){
					isDragging = true;
					if(ui.helper){
						ui.helper.mousemove(function(e){
							if(isDragging && typeof(scope.onDropDrag) === 'function'){
								var drop = $(ui.draggable).data('drop');
								scope.onDropDrag(drop, e);
							}
						});
					}
				},
				out: function(e, ui){
					isDragging = false;
				}
			};

			options = $.extend({}, defaultOptions, options);
			elm.droppable(options);
		}
	};
}]);

angular.module('app').filter('filterObj', [function() {
	return function(input, property, search) {

		if(!property || !search || search.length < 1){
			return input;
		}
		var result = {};

		var getPathValue = function(obj, path){
			var value = obj;
			var parts = path.split('.');

			for(var i=0; i<parts.length; i++){
				value = value[parts[i]];
			}

			return value;
		};

		for(var itemKey in input){
			var item = input[itemKey];
			if(typeof(getPathValue(item, property) === 'string') &&
				getPathValue(item, property).toLowerCase().indexOf(search.toLowerCase()) >= 0){
				result[itemKey] = item;
			}
		}
		return result;
	};
}]);

angular.module('app').filter('filterPkgList', [function() {
	return function(input, criteria) {

		if(!input){
			return input;
		}

		var result = {};

		for(var id in input){
			var pkg = input[id];
			
			// all
			if(criteria === 'all' && pkg.type == 'drop'){
				result[id] = pkg;
			}
			
			// free
			if(criteria === 'free' && pkg.meta.price == 0 && pkg.type == 'drop'){
				result[id] = pkg;
			}
			
			// non free
			if(criteria === 'paid' && pkg.meta.price > 0 && pkg.type == 'drop'){
				result[id] = pkg;
			}
			
			// integrations
			if(criteria === 'integrations' && pkg.type == 'integration'){
				result[id] = pkg;
			}
		}

		return result;
	};
}]);

angular.module('app').filter('filterPaginate', [function() {
	return function(input, pagObj, limit){
		limit = limit || 10;
		if(!pagObj) return input;

		// set first page
		if(!pagObj.currentPage) pagObj.currentPage = 1;
		if(!pagObj.next) pagObj.next = function(){
			if(pagObj.currentPage < pagObj.pages) pagObj.currentPage++;
		};
		if(!pagObj.prev) pagObj.prev = function(){
			if(pagObj.currentPage > 1) pagObj.currentPage--;
		};
		if(!pagObj.isLastPage) pagObj.isLastPage = function(){
			return (pagObj.currentPage === pagObj.pages);
		};
		if(!pagObj.isFirstPage) pagObj.isFirstPage = function(){
			return (pagObj.currentPage === 1);
		};
		if(!pagObj.canPaginate) pagObj.canPaginate = function(){
			return (pagObj.pages > 1);
		};
		if(!pagObj.getPageNumbers) pagObj.getPageNumbers = function(){
			var nums = [];
			var dist = 4;
			var start = 0;
			var end = pagObj.pages;

			if(pagObj.currentPage - dist > 0){ 
				start = pagObj.currentPage - dist;
				end = pagObj.currentPage + dist + 1;
			}
			else{ 
				start = 0; 
				end = dist*2 + 1;
			}

			if(end > pagObj.pages){ 
				end = pagObj.pages; start = Math.max(pagObj.pages - 2*dist - 1, 0);
			}

			for(var i=start; i<end; i++){
				nums.push(i+1);
			}
			return nums;
		};


		var itemCount = Object.keys(input).length;
		
		// if the item count has changed, reset the pagination
		if(!pagObj.itemCount || pagObj.itemCount !== itemCount){
			pagObj.itemCount = itemCount;
			pagObj.currentPage = 1;
			pagObj.pages = Math.ceil(itemCount / limit);
		}

		var idx = 0;
		var res = (angular.isObject(input) ? {} : []);

		for(var prop in input){
			if(idx >= (pagObj.currentPage-1) * limit && idx < pagObj.currentPage * limit){
				res[prop] = input[prop];
			}
			idx++;
		}
		return res;
	};
}]);

angular.module('app').filter('orderObjectByKey', function () {
	return function (obj) {
		if (!angular.isObject(obj)) return obj;

		var keymap = [];

		for(var key in obj) {
			if(obj.hasOwnProperty(key)){
				keymap.push(key);
			}
		}

		// sort the keymap
		keymap.sort();
		var buffer = {};
		
		for(var key in obj) {
			if(obj.hasOwnProperty(key)){
				buffer[key] = obj[key];
				delete obj[key];
			}
		}
		
		// restore
		for(var i=0; i<keymap.length; i++){
			obj[keymap[i]] = buffer[keymap[i]];
		}

		return obj;
	};
});

angular.module('app').filter('toArray', function () {
	return function (obj) {
		if (!(obj instanceof Object)) {
			return obj;
		}

		return Object.keys(obj).map(function (key) {
			return Object.defineProperty(obj[key], '$key', {__proto__: null, value: key});
		});
	};
});
