/**
 * Controller for our Dropper Upgrades
 */
angular.element(document).ready(function() {

	// get the angularjs injector
	angular.injector(['app']).invoke(['upgrade', 'core', 'message', 'utils', function(upgrader, core, message, utils){

		// ugprade that does nothing
		var emptyUpgrade = function(settings) {
			return settings;
		};

		/**
		 * 100.0 - 100.1
		 * This upgrade refactores the Drops, so that they work with _internal objects,
		 * No worries, it's cleaner :)
		 */
		upgrader.addDropperUpgrade("100.0", "100.1", function(settings){
	
			for(var i=0; i<settings.drops.length; i++){
	
				var drop = settings.drops[i];
	
				if(!drop._internal){
	
					// new internal object		
					drop._internal = {
						name: drop.name,
						replaceMethod: drop.replaceMethod,
						selector: drop.what,
						customCondition: drop.customCondition ? drop.customCondition : false,
						dropId: drop.template,
						guid: drop._guid,
						version: drop._version,
						visibility: drop.visibility,
						condition: drop.condition ? drop.condition : "",
						conditionalSettings: drop.conditionalSettings ? drop.conditionalSettings : null
					};
	
					delete drop.name;
					delete drop.replaceMethod;
					delete drop.what;
					delete drop.customCondition;
					delete drop.template;
					delete drop._guid;
					delete drop._version;
					delete drop.visibility;
					delete drop.condition;
					delete drop.conditionalSettings;
	
					if(drop.dropItems && drop.dropItems.length > 0){
						for(var j=0; j<drop.dropItems.length; j++){
							var dropItem = drop.dropItems[j];
							if(!dropItem._internal){
								dropItem._internal = {
									visibility: dropItem.visibility,
									name: dropItem.name
								}
							}
						}
					}
				}
			}
	
			return settings; 
		});
	
		/**
		 * 100.1 - 100.2
		 * This Upgrade removes some old Properties from your Drops that were forgotten in 100.1 upgrade :)
		 */
		upgrader.addDropperUpgrade("100.1", "100.2", function(settings){
	
			for(var i=0; i<settings.drops.length; i++){
	
				var drop = settings.drops[i];
				// upgrade from 100.0 to 100.1 forgot to remove the stylesheets and scripts array after moving them to the drop._internal object
				if (upgrader.dropHasOption(drop._internal.dropId, 'stylesheets') === false) delete drop.stylesheets;
				if (upgrader.dropHasOption(drop._internal.dropId, 'scripts') === false) delete drop.scripts;
	
				// upgrade from 100.0 to 100.1 forgot to remove the name and visibility properties after moving them to the dropItem._internal object
				if (upgrader.dropItemHasOption(drop._internal.dropId, 'name') === false && drop.dropItems instanceof Array) {
					for (var j=0; j<drop.dropItems.length; j++) {
						delete drop.dropItems[j].name;
					}
				}
	
				if (upgrader.dropItemHasOption(drop._internal.dropId, 'visibility') === false && drop.dropItems instanceof Array) {
					for (var k=0; k<drop.dropItems.length; k++) {
						delete drop.dropItems[k].visibility;
					}
				}
	
			}
			return settings;
		});
	
		/**
		 * 100.2 - 100.3
		 * This Upgrade adds equality options to conditionals
		 */
		upgrader.addDropperUpgrade("100.2", "100.3", function(settings){
	
			for(var i=0; i<settings.drops.length; i++){
				var drop = settings.drops[i];
				if(angular.isArray(drop._internal.conditionalSettings)){
					for(var j=0; j<drop._internal.conditionalSettings.length; j++){
						var condSetting = drop._internal.conditionalSettings[j];
						if(typeof(condSetting.equality) === 'undefined'){
							condSetting.equality = true;
						}
					}
				}
			}
			return settings;
		});
	
		/**
		 * 100.3 - 100.4
		 * This Upgrade rewrites the manually set smarty condition to use the conditional Smarty source
		 */
		upgrader.addDropperUpgrade("100.3", "100.4", function(settings){
	
			for(var i=0; i<settings.drops.length; i++){
				var drop = settings.drops[i];
				if(drop._internal.customCondition && drop._internal.customCondition === true && angular.isString(drop._internal.condition)){
					drop._internal.conditionalSettings = [
						{
							conditional: "Smarty",
							setting: {
								choices: {
									expression: drop._internal.condition
								},
								equality: true,
								operand: "and"
							}
						}
					]
				}
	
				if(typeof(drop._internal.customCondition) !== 'undefined'){
					delete drop._internal.customCondition;
				}
			}
			return settings;
		});
	
		/**
		 * 100.4 - 100.5
		 * This Upgrade does nothing to the data but fixes a DateTime::getTimestamp problem with PHP < 5.3
		 */
		upgrader.addDropperUpgrade("100.4", "100.5", emptyUpgrade);
	
		/**
		 * 100.5 - 100.6
		 * This upgrade does nothing. The code fixes an encoding problem with umlauts in languages.
		 */
		upgrader.addDropperUpgrade("100.5", "100.6", emptyUpgrade);

		/**
		 * 100.6 - 100.7
		 * This upgrade does nothing. The code improves performance fixes and a better media library.
		 */
		upgrader.addDropperUpgrade("100.6", "100.7", emptyUpgrade);
	
		/**
		 * 100.7 - 100.8
		 * This upgrade does nothing. Htaccess rules for kk_upload.php were changed to reflect changes in 3.20 update
		 */
		upgrader.addDropperUpgrade("100.7", "100.8", emptyUpgrade);

		/**
		 * 100.8 - 100.9
		 * Upgrade to package management and settings
		 */
		upgrader.addDropperUpgrade("100.8", "100.9", emptyUpgrade);		

		/**
		 * 100.9 - 100.10
		 * Fixes ioncube issues with PHP 5.4
		 */
		upgrader.addDropperUpgrade("100.9", "100.10", emptyUpgrade);

		/**
		 * 100.10 - 100.11
		 * This upgrade, among other things, adds configurable repositories to Dropper
		 */
		upgrader.addDropperUpgrade("100.10", "100.11", function(settings){
			settings.backupLifetime = "60";
			return settings;
		});


		/**
		 * 100.11 - 100.12
		 * This upgrade removes the inline CSS setting, wich is of no use nowadays,
		 * but also was in trouble with the 4.03 shop upgrade
		 */
		upgrader.addDropperUpgrade("100.11", "100.12", function(settings){
			if(typeof(settings.inlineCSS) !== 'undefined') delete settings.inlineCSS;
			return settings;
		});

		/**
		 * 100.12 - 100.13
		 * Updated Article Conditional to work with child articles
		 */
		upgrader.addDropperUpgrade("100.12", "100.13", function(settings){
			if (settings && settings.drops) utils.refreshConditionsDeep(settings.drops);
			return settings;
		});

		/**
		 * 100.13 - 100.14
		 * Defer JS and inlining of small files, Template Integrations
		 */
		upgrader.addDropperUpgrade("100.13", "100.14", function(settings){
			settings.deferJS = false;
			settings.inlineTypes = [];
			settings.inlineLimit = "20";
			
			// migrate template base to integration selection
			switch (settings.templateBase) {
				case 'tiny':
					settings.integration = 'INT_Tiny';
				break;
				case 'evo':
					settings.integration = 'INT_Evo';
				break;
				default:
					settings.integration = '_auto';
				break;
			}
			
			// remove template base setting, as it is not used anymore
			delete settings.templateBase;

			// set a guid on every _internal object
			var setInternalGuids = function(obj) {
	
				if (angular.isObject(obj)) {
					for (var prop in obj) {
						setInternalGuids(obj[prop]);
					}
				}
				
				if (obj && obj._internal && !angular.isString(obj._internal.guid)) {
					// create a guid
					obj['_internal']['guid'] = utils.generateGuid();
				}
			};
			
			if(settings && settings.drops && settings.drops.length){
				for(var i=0;i<settings.drops.length;i++){
					var drop = settings.drops[i];
					setInternalGuids(drop);
				}
			}

			// refresh conditionals as the PageType conditional has changed
			if (settings && settings.drops) utils.refreshConditionsDeep(settings.drops);

			return settings;
		});

		/**
		 * 100.14 - 100.15.0
		 * Introduces Ghost zones that do not destroy the template structure
		 */
		upgrader.addDropperUpgrade("100.14", "100.15.0", function(settings) {
			message.successHTML('Willkommen zu Dropper 100.15', 'Es gibt viel Neues zu entdecken! Hier geht es zu den <a target="_blank" href="https://kreativkonzentrat.de/Dropper-10015-Release-Notes">Dropper Release Notes</a>.', 0);
			return settings;
		});
		upgrader.addDropperUpgrade("100.14.0", "100.15.0", function(settings) {
			message.successHTML('Willkommen zu Dropper 100.15', 'Es gibt viel Neues zu entdecken! Hier geht es zu den <a target="_blank" href="https://kreativkonzentrat.de/Dropper-10015-Release-Notes">Dropper Release Notes</a>.', 0);
			return settings;
		});
		
		/**
		 * 100.15.0 - 100.15.1
		 * Fixes issues with cached smarty evaluation under older smarty versions
		 * Fixes issues with search in datasources on older sql versions
		 */
		upgrader.addDropperUpgrade("100.15.0", "100.15.1", emptyUpgrade);

		/**
		 * 100.15.1 - 100.15.2
		 * Fix smarty scope issues for older smarty versions (for real)
		 * Fix Datepicker labels in wrong order
		 */
		upgrader.addDropperUpgrade("100.15.1", "100.15.2", emptyUpgrade);
		
		/**
		 * 100.15.2 - 100.15.3
		 * Fix an issue where licenses could not be fetched because of wrongly configured http_build_query seperators
		 * Enhance the category conditional to be able to include subcategories of the given ones
		 */
		upgrader.addDropperUpgrade("100.15.2", "100.15.3", emptyUpgrade);

		/**
		 * 100.15.3 - 100.15.4
		 * Fix resource lazy loading under ajax calls was broken
		 * Show svg file previews in media gallery
		 * Automatic redirect on frontend / backend protocol mismatch
		 */
		upgrader.addDropperUpgrade("100.15.3", "100.15.4", emptyUpgrade);

		/**
		 * 100.15.4 - 100.15.5
		 * Fix Drops with empty names
		 */
		upgrader.addDropperUpgrade("100.15.4", "100.15.5", function(settings) {
			
			for (var i=0; i<settings.drops.length; i++) {
				var drop = settings.drops[i];
				if (drop._internal && !drop._internal.name) drop._internal.name = "Unbenannt";
			}
			return settings;
			
		});
		
		/**
		 * 100.15.5 - 100.15.6
		 * Fix some IO bugs
		 * Show latest changelog on updates in packagemanager
		 */
		upgrader.addDropperUpgrade("100.15.5", "100.15.6", emptyUpgrade);

		/**
		 * 100.15.6 - 100.15.7
		 * Fix contenteditable
		 * Fix image offset in Chromium
		 * Drop-Datasource support
		 * Unset Drop assignment after render to not bloat JTL-Debug
		 * Skip insertion API is public
		 */
		upgrader.addDropperUpgrade("100.15.6", "100.15.7", emptyUpgrade);

		/**
		 * 100.15.7 - 100.15.8
		 * Fix possible XSS bug with Drop IO interface
		 */
		upgrader.addDropperUpgrade("100.15.7", "100.15.8", emptyUpgrade);

		/**
		 * 100.15.8 - 100.15.9
		 * Fix possible uploader vulnerability
		 */
		upgrader.addDropperUpgrade("100.15.8", "100.15.9", emptyUpgrade);

		/**
		 * 100.15.9 - 100.15.10
		 * Moved uploader into frontend, using improved security validation
		 */
		upgrader.addDropperUpgrade("100.15.9", "100.15.10", emptyUpgrade);

		/**
		 * 100.15.10 - 100.15.11
		 * Improve suspicious file checks
		 */
		upgrader.addDropperUpgrade("100.15.10", "100.15.11", emptyUpgrade);

		/**
		 * 100.15.11 - 100.15.12
		 * More security checkups
		 */
		upgrader.addDropperUpgrade("100.15.11", "100.15.12", emptyUpgrade);

		/**
		 * 100.15.12 - 100.15.13
		 * Fix path handling when shop is served from subdirectory
		 */
		upgrader.addDropperUpgrade("100.15.12", "100.15.13", emptyUpgrade);

	}]);
});