angular.module('app').controller('MainViewCtrl', ['$scope', '$rootScope', '$timeout', '$templateCache', 'core', 'message', 'environment', 'datasource', 'upgrade', 'modal', 'editWindow', 'media', 'packageManager', 'utils', 'rootScopeLegacy', 
	function($scope, $rootScope, $timeout, $templateCache, core, message, environment, datasource, upgrade, modal, editWindow, media, packageManager, utils, rootScopeLegacy){

	/*********************************************
	 Whenever we load this should be done
	 ********************************************/
	var load = function(){
		$rootScope.activeDrop = undefined;
		$rootScope.activeTemplate = undefined;
	};
	core.fn.onReload(load);
	load();

	// Expose the scope to the outer word (for debugging purposes)
	window.mainViewScope = $scope;

	// lets handle this a bit differntly, as we got a resolved promise returned
	// this is because we have to wait to core to load and do this in the router application.js
	$scope.core = core;

	/*********************************************
	 Empty the Template Cache, we don't want it
	 ********************************************/
	$templateCache.removeAll();

	$rootScope.activeDrop = undefined;
	$rootScope.activeTemplate = undefined;
	$rootScope.defaultGroup = "Ungruppiert";
	$rootScope.linkMapFromTo = {};
	$rootScope.linkMapToFrom = {};
	$scope.rootScope = $rootScope;

	$scope.environment = environment;
	$scope.packageManager = packageManager;
	
	$scope.saveLabel = "Gespeichert";
	$scope.onOffOptions = [{ label: 'An', value: true }, { label: 'Aus', value: false }];
	$scope.overviewOpen = true;
	$scope.groups = {};
	$scope.editMode = false;
	$scope.licenceKeyInput = "";
	$scope.dropInfo = {};
	$rootScope.activeDropLinks = [];
	$scope.saveInProgress = false;
	$scope.infotab = "infos";
	$scope.dropSearch = "";
	$scope.storeSearch = "";
	$scope.dashboardView = { type : "installed", filter: "all" };
	$scope.storePagination = {};
	$scope.dropPagination = {};
	
	$scope.settingsConfig = [
		{name: 'Unlock Key', description: 'Dropper Plugin unlocken, sodass eigene Drops genutzt werden können', type: 'text', property: 'key'}
	];

	$scope.generalDropConfigDefaults = {
		_internal: {
			selector: '',
			replaceMethod: '',
			dropId: 'Plain',
			visibility: 'active'
		}
	};
	
	/**
	 * Recursively loops over the config properties and looks in the obj if we have got
	 * string properties that should be multilang.
	 */
	var _updateMultiLangFields = function(obj, cfg){
		var changedArr = [];

		for(var i=0; i<cfg.length; i++){
			var option = cfg[i];
			if(angular.isString(obj[option.property]) && (option.multilanguage === true || option.multilanguage === "true")){
				console.log("changing drop property: ", obj, "from value:", obj[option.property], "at property:", option.property, "to multilanguage");
				changedArr.push({ item: obj, prop: option.property });
				var oldVal = obj[option.property];
				obj[option.property] = { _multilanguage: true };
				obj[option.property][environment.languageDefault.cISO] = oldVal;
			}
			if(angular.isArray(obj[option.property]) && option.type === 'list'){
				changedArr.concat(_updateMultiLangFields(obj[option.property], option.itemSettings));
			}
		}
		return changedArr;
	};

	/**********************************************
	 Scope Methods
	 *********************************************/
	$scope.updateMultiLangFields = function(){
		var changed = 0;
		var changedArr = [];

		for(var dropID in core.settings.drops){
			var drop = core.settings.drops[dropID];
			if(!drop._internal || !drop._internal.dropId){
				console.log("Could not update Multilang Fields because no _internal or dropId set", drop);
				continue;
			}

			var dropSetting = core.templates[drop._internal.dropId];
			if(dropSetting && dropSetting.config) changedArr.concat(_updateMultiLangFields(drop, dropSetting.config));
			if(dropSetting && dropSetting.settings) changedArr.concat(_updateMultiLangFields(drop, dropSetting.settings));
		}
		
		if(changedArr.length > 0){
			message.info(changed + ' Objekte wurden mit Mehrsprachigkeit nachgerüstet.', 'Bitte Speichern um Änderungen zu übernehmen.');
			console.log('multilang update: ', changedArr, false);
		}
	};

	/**
	 * Generates a unique Identifier
	 */
	$rootScope.generateGuid = function(){
		return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
			return v.toString(16);
		});
	};

	/**
	 * Counts the properties of an object
	 */
	$rootScope.countObject = function(obj){
		if(!obj) return 0;
		return Object.keys(obj).length;
	};
	
	/**
	 * AngjularJS object might contain Hash Keys. This function removes them
	 * so that we can have a clean output to save.
	 */
	$scope.removeHashes = function(obj){
		if(obj['$$hashKey']){
			delete obj['$$hashKey'];
		}

		for(var prop in obj){
			if(typeof(obj[prop]) === 'array' || typeof(obj[prop]) === 'object'){
				$scope.removeHashes(obj[prop]);
			}
		}
	};

	/**
	 * Shows the Drop Dialog with a given Tab and Name
	 * @drop - the Drop object
	 * @name - the Name visible in the Dialog
	 * @e - the Event Object
	 * @tab - name of the Tab to open
	 * @scope - 'installed' or 'shop'
	 */
	$rootScope.showDropInfo = function(drop, name, tab, e){
		if(e){
			e.preventDefault();
			e.stopPropagation();
		}

		// build info object
		var dropInfo = {
			drop: drop,
			name: name,
			tab: (tab && tab.length > 0) ? tab : "infos"
		};

		// show the dialog
		modal.show('DropInfoModal', null, {"dropInfo": dropInfo});
	};

	$scope.showImportExport = function(type){
		if(!type){
			type = "export";
		}
		modal.show('ImportExportModal', null, {"type": type});
	};

	$scope.showSelectorPicker = function(){
		modal.show('SelectorPickerModal', null);
	};

	$scope.showSettings = function(){
		modal.show('SettingsModal', function(){
		});
	};

	$scope.getUpgradesCount = function(){
		return Object.keys(core.packages.upgradeable).length;
	};

	$scope.showPackageManager = function(mode, pkg){
		modal.show('PackageManagerModal', null, {"overviewMode": null, "mode": mode || "overview", "package" : pkg || null});
	};
	
	$scope.checkForcedUpgrades = function() {
		if(packageManager.hasForcedUpgrades()){
			modal.show('PackageManagerModal', null, {"overviewMode": "forcedUpgrades", "mode": "overview", "package": null});
		}
	};

	/**
	 * Retrieves the List of available Backups
	 */
	$scope.listBackups = function(){
		datasource.get('SaveHistory', function(res, err){
			if(res){
				$scope.$apply(function(){
					$scope.saveHistory = res;
				});
			}else{
				message.error('Fehler beim Laden der Backups.', 'Die Liste von Backups konnte nicht geholt werden.');
			}
		}, true);
	};

	/**
	 * Restores a Backup
	 */
	$scope.restoreBackup = function(backup){
		console.log('trying restoring: ', backup.file);
		$.ajax({
			type: 'POST',
			url: core.shopURL,
			data: { "kk_dropper_admin_pass": core.adminSecret, "kk_dropper_get_backup": backup.timestamp },
			dataType: 'text',
			success: function(res){
				res = angular.fromJson(res);
				console.log('json', res);
				
				$rootScope.$apply(function(){
					$rootScope.activeDrop = undefined;
					// run upgrade manager
					core.settings = res;
					$scope.setDirtySaveState(true);
					$rootScope.verifyChanges();
					message.success('Das Backup vom ' + backup.label + ' wurde erfolgreich geladen', 'Bitte speichern um die geladenen Backup-Daten zu übernehmen.', 0);
				});
			},
			error: function(err, reason){
				message.error('Konnte das Backup von ' + backup.file + ' nicht wiederherstellen!',  'Fehler: ' + reason + '.\n' + JSON.stringify(err.responseText));
				console.log(err.responseText);
			}
		});
	};

	/**
	 * Shows the Drop Overview Page
	 */
	$scope.showOverview = function(){
		$scope.overviewOpen = true;
		$('#drop-select').removeClass('select-offscreen');
		$('html').css('overflow', 'hidden');
	};

	/**
	 * Hides the Drop Overview Page
	 */
	$scope.hideOverview = function(){
		$scope.overviewOpen = false;
		$('html').css('overflow', '');
		$('#drop-select').addClass('select-offscreen');
	};

	/**
	 * Toggles the Drop Overview Page
	 */
	$scope.toggleOverview = function(){
		return $scope.overviewOpen ? $scope.hideOverview() : $scope.showOverview();
	};

	$scope.getLicenceTypeName = function(){
		var types = {
			"kk_dropper_bronze" : "Bronze",
			"kk_dropper_silber" : "Silber",
			"kk_dropper_gold" : "Gold",
			"kk_dropper_super" : "Dev"
		};
		return types[core.dropperLicense.item];
	};

	$scope.openGroup = function(group){
		// close other groups
		for(var groupName in $scope.groups){
			var otherGroup = $scope.groups[groupName];
			if(otherGroup != group) { 
				$scope.closeGroup(otherGroup);
			}
		}

		// open self group
		group.open = true;
	};

	$scope.closeGroup = function(group){
		group.open = false;
	};

	$scope.toggleGroup = function(group){
		if(group.open){
			$scope.closeGroup(group);
		}else{
			$scope.openGroup(group);
		}
	};

	/**
	 * Generic Remove function. Determines weather it is a Drop
	 */
	$scope.removeItem = function(){
		if($scope.toRemove && core.settings.drops.indexOf($scope.toRemove) >= 0){
			// remove a drop
			$scope.removeDrop($scope.toRemove);
		}
		$('#remove-popup').removeClass('in');
	};

	/**
	 * Shows a Popup that asks if the object shell be deleted
	 */
	$rootScope.toggleRemove = function(item, e, callback){
		var jPopup = $('#remove-popup');

		if(e && e.target){
			e.preventDefault();
			e.stopPropagation();
			var jTarget = $(e.target);
			// try to position the popup to the given target
			var pos = jTarget.offset();
			jPopup.css({ left: (pos.left + jTarget.width()/2 - jPopup.width()/2) + 'px', top: (pos.top + jTarget.height()/2) + 'px'});
		}

		if(item && e){
			e.stopPropagation();
		}

		if(item){
			$scope.toRemove = item;
			jPopup.addClass('in');
		}else{
			jPopup.removeClass('in');
		}
	};

	/**
	 * Checks if the given Template is able to create new Drops
	 * @template - the template of the drop
	 */
	$rootScope.canAddDrop = function(template){
		if( !template || 
			!core.dropLicenses || 
			!core.dropLicenses[template] || 
			!core.templates[template]){
			return false;
		}
		return core.dropLicenses[template].isValid;
	};

	/**
	 * Given a basename, this function tries to generate a new 
	 * unique name by adding a number at the end
	 */
	$rootScope.generateUniqueDropName = function(baseName){
		// try to generate a meaningfull new name for it
		var newName = baseName;
		var nameCounter = 1;

		for(var i=0; i<core.settings.drops.length; i++){
			var loopName = core.settings.drops[i]._internal.name;
			if(loopName === newName){
				var loopNumber = loopName.match(/\d+/g);
				if(loopNumber && loopNumber.length){
					nameCounter = parseInt(loopNumber[0], 10) + 1;
					newName = newName.replace(/\d+/g, nameCounter);
				} else {
					nameCounter ++;
					newName = baseName + nameCounter;
				}
				i=0;
			}
		}
		return newName;
	};

	/**
	 * Add a new Drop
	 * @template - the template for the new drop
	 * @activate - immediately activates the new Drop
	 */
	$rootScope.addDrop = function(template, activate){

		if(typeof(activate) == 'undefined'){
			activate = true;
		}

		// perform a check to see if we can add this drop
		if(!$scope.canAddDrop(template)){
			// show the licence dialog
			$scope.showDropInfo(template, template, 'licence');
			return null;
		}

		var newName = $rootScope.generateUniqueDropName(core.templates[template].name);

		var newDrop = {
			_internal: {
				name: newName,
				dropId: template,
				guid: $scope.generateGuid(),
				version : (template && core.templates[template] && core.templates[template].version) ? core.templates[template].version : 'not_versioned'
			}
		};

		$scope.setTemplate(template, newDrop);
		core.settings.drops.push(newDrop);

		// activate it
		if(activate){
			$scope.activateDrop(newDrop);
		}
		return newDrop;
	};

	/**
	 * Adds a new Drop and links it to a given other drop
	 * @template - the Template for the new Drop
	 * @drop - the Drop to link into
	 * @property - the Drop Property under which the link is stored
	 */
	$rootScope.addAndLinkDrop = function(template, drop, property){
		console.log('add and link:', template, drop, property);
		var newDrop = $scope.addDrop(template);
		newDrop._internal.selector = '';
		newDrop._internal.replaceMethod = '';
		drop[property] = newDrop._internal.guid;
	};

	/**
	 * Asks the user to choose a Drop and creates it. Then returns the Drop instance back inside a callback
	 */
	$rootScope.getNewDropUserSelect = function(callback /* drop */, activate){
		modal.show('NewDropModal', function(template){
			if(template){
				var shouldActivate = ((typeof activate === 'undefined' || activate === true) ? true : false);
				var drop = $rootScope.addDrop(template, shouldActivate);
				if(angular.isFunction(callback)){
					callback(drop);
				}
			}
		});
	};

	/**
	 * Opens up the Media Gallery
	 */
	$rootScope.showMediaGallery = function(){
		media.select();
	};

	/**
	 * Gets a Drop by GUID
	 */
	$rootScope.getDropByGuid = function(guid){
		for(var idx in core.settings.drops){
			var drop = core.settings.drops[idx];
			if(drop._internal && drop._internal.guid === guid){
				return drop;
			}
		}
	};

	/**
	 * Builds two maps that describe the linking between drops:
	 * $scope.mapFromTo - drop | list of drops it links in its data
	 * $scope.mapToFrom - drop | list of drops linked to it from thier data
	 */
	$scope.buildLinkMap = function(drop){
		// mapFromTo: { 'guid-123' : [{'guid-321', name:'abc'}] }
		// mapToFrom: { 'guid-321' : [{'guid-123', name:'xyz'}] }
		var mapFromTo = $rootScope.linkMapFromTo;
		var mapToFrom = $rootScope.linkMapToFrom;

		// helper function that checks if a given guid already exists in a link array
		var exists = function(guid, links){
			if(!links || !angular.isArray(links) || !links.length) return false;
			for(var i=0; i<links.length; i++){
				if(links[i].guid === guid) return true;
			}
			return false;
		};

		// helper function that analyzes a given drop for other linked drops
		// by searching the JSON stringified object for references.
		var updateDropLinks = function(drop){
			if(!drop || !drop._internal || !drop._internal.guid) return;

			var dropData = JSON.stringify(drop);
			var linked = $scope.getDropsFromText(dropData);

			if(typeof(mapFromTo[drop._internal.guid]) === 'undefined') mapFromTo[drop._internal.guid] = [];
			if(typeof(mapToFrom[drop._internal.guid]) === 'undefined') mapToFrom[drop._internal.guid] = [];

			for(var idl = 0; idl < linked.length; idl++){
				var linkedWrapper = linked[idl];

				if(!linkedWrapper.found) continue;

				var link = linkedWrapper.drop;

				if(typeof(mapToFrom[link._internal.guid]) === 'undefined' || !angular.isArray(mapToFrom[link._internal.guid])){
					mapToFrom[link._internal.guid] = [];
				}

				if(drop._internal.guid !== link._internal.guid){
					if(!exists(link._internal.guid, mapFromTo[drop._internal.guid])) mapFromTo[drop._internal.guid].push({ name: link._internal.name || '', guid: link._internal.guid });
					if(!exists(drop._internal.guid, mapToFrom[link._internal.guid])) mapToFrom[link._internal.guid].push({ name: drop._internal.name || '', guid: drop._internal.guid });
				}
			}			
		};

		// we have a changed drop given, just update its internal references
		if(drop){
			mapFromTo[drop._internal.guid] = [];
			updateDropLinks(drop);
		}
		// if no drop was given update the whole map
		else {
			// cleanup old results
			for(var prop in $rootScope.linkMapFromTo){
				delete $rootScope.linkMapFromTo[prop];
			}
			for(var prop in $rootScope.linkMapToFrom){
				delete $rootScope.linkMapToFrom[prop];
			}

			// get new links
			for(var idx = 0; idx < core.settings.drops.length; idx++){
				var drop = core.settings.drops[idx];

				if(!drop || !drop._internal || !drop._internal.dropId){
					continue;
				}
				updateDropLinks(drop);
			}
		}
	};


	/**
	 * Get outbound links from string properties of the given object
	 */
	function _getOutboundLinks(obj){

		if(typeof(obj) === 'undefined' || obj === null){
			return [];
		}

		var results = [];

		for(var prop in obj){
			if(prop === "$$hashKey" || prop === "_internal") continue;

			// value of the property
			var propValue = obj[prop];

			// check what it is, treat it differently
			if(angular.isString(propValue)){
				results = results.concat($scope.getDropsFromText(propValue));
			}
			else if(angular.isArray(propValue)){
				for(var i=0; i<propValue.length; i++){
					results = results.concat(_getOutboundLinks(propValue[i]));
				}
			}
			else if(angular.isObject(propValue)){
				results = results.concat(_getOutboundLinks(propValue));
			}
		}
		return results;
	}

	/**
	 * Get all outgoing Links to other drops for the given Drop
	 */
	$rootScope.getOutboundLinks = function(drop){
		if(!drop) return [];
		var results = _getOutboundLinks(drop);

		// work them out
		var realResults = [];
		for(var i=0; i<results.length; i++){
			if(results[i].found === true){
				realResults.push(results[i].drop);
			}
		}
		return realResults;
	};

	/**
	 * Activates a linked Drop
	 */
	$scope.activateLink = function(link, e){
		if(link.drop){
			$scope.activateDrop(link.drop);
		}
	};

	/**
	 * Removes a Drop from the Settings List
	 * @drop - the Drop Object to remove
	 */
	$rootScope.removeDrop = function(drop){

		// check if the drop exists in the settings
		if(core.settings.drops.indexOf(drop) < 0){
			return false;
		}

		// close all editing windows
		if(drop === $rootScope.activeDrop){
			editWindow.closeAll();
		}

		// remove it from the settings
		core.settings.drops.splice(core.settings.drops.indexOf(drop), 1);
		if($rootScope.activeDrop == drop){
			// unset active drop, but only if this one was the active drop :)
			$rootScope.activeDrop = undefined;
		}
		return true;
	};

	/**
	 * Clone a given Drop and generate a new GUID
	 */
	$scope.duplicateDrop = function(drop, e){
		if(e){
			e.stopPropagation();
		}
		modal.show('DuplicateModal', function(res){
			
		}, {"drop": drop});
	};

	/**
	 * Sets the active Drop (which is being edited)
	 * @param  {object} drop - the Drop
	 */
	$rootScope.activateDrop = function(drop, e){

		if(e){
			e.preventDefault();
			e.stopPropagation();
		}

		if(typeof(drop) === 'string'){
			// this seems to be a guid, let's try to get a drop from it
			drop = $scope.getDropByGuid(drop);
		}

		if(!drop) return;

		if(!core.templates[drop._internal.dropId]){
			// no template found for this drop, show infos on click
			$scope.showDropInfo(drop._internal.dropId, drop._internal.dropId, 'licence');
			return;
		}

		if($rootScope.activeDrop != drop){
			// no drop open yet
			$scope.hideOverview();
			$rootScope.activeDrop = drop;
			$rootScope.activeTemplate = core.templates[drop._internal.dropId];

			editWindow.show(false, 'DropEditWindow', {
				'activeDrop' : $rootScope.activeDrop,
				'activeTemplate' : $rootScope.activeTemplate
			});
			return;
		}
		return false;
	};

	$scope.setTemplate = function(template, drop){
		if(!drop){
			drop = $rootScope.activeDrop;
		}
		drop._internal.dropId = template;

		// set defaults
		$scope.setDefaultDropConfig(drop);
	};

	$scope.setDefaultDropConfig = function(drop){
		if(drop){
			var dropBefore = $.extend(true, {}, drop);
			$.extend(true, drop, $scope.generalDropConfigDefaults, dropBefore);

			if(drop._internal.dropId){
				var currentTemplate = core.templates[drop._internal.dropId];

				if(currentTemplate.defaults){
					console.log('merging default drop options');
					$.extend(true, drop, currentTemplate.defaults);
				}
			}
		}
		return drop;
	};

	$scope.truncate = function(text, length){
		if(text){
			return text.length > length ? text.substr(0, length) + '...' : text;
		}
		return "";
	};

	/**
	 * Moves a given drop to the given groupName
	 */
	$scope.moveDropToGroup = function(drop, groupName){

		console.log('moving drop into group:', drop, groupName);

		if(groupName === 'Ungruppiert') groupName = '';
		else groupName = groupName + ': ';

		var newName = drop._internal.name;
		
		var parts = newName.split(':');
		if(parts && parts.length > 1){
			// remove first, this is the old groupname
			parts.shift();
			newName = parts.join(':').replace(/^\s+/,"");
		}

		newName = groupName + newName;

		$timeout(function(){
			if (drop._internal.name !== newName) {
				drop._internal.name = newName;
				$scope.updateGroups();
				$scope.setDirtySaveState(true);
			}
		});
	};

	$scope.updateGroups = function(){
		var groups = $scope.groups;
		var groupName;
		
		// empty the groups		
		for(groupName in groups){
			var group = groups[groupName];
			group.splice(0, group.length);
		}
		
		// add drops to the groups
		for(var i=0; i<core.settings.drops.length; i++){
			var drop = core.settings.drops[i];
			groupName = $rootScope.defaultGroup;

			if(!drop || !drop._internal || !drop._internal.name){
				continue;
			}

			if(drop._internal.name.indexOf(':') > 0){
				groupName = $rootScope.groupLabel(drop._internal.name);
				if(!$scope.groups[groupName]){
					$scope.groups[groupName] = [];
				}
				$scope.groups[groupName].push(drop);
			}else{
				if(!$scope.groups[$rootScope.defaultGroup]){
					$scope.groups[$rootScope.defaultGroup] = [];
				}
				$scope.groups[$rootScope.defaultGroup].push(drop);
			}
		}
		
		// remove empty groups
		for(var gName in groups){
			if(!groups[gName].length) delete groups[gName];
			else groups[gName].groupName = gName;
		}
		
	};

	$rootScope.ungroupLabel = function(label){
		if(label.indexOf(':') > 0){
			// remove group seperator and trim left
			return label.substr(label.indexOf(':') + 1).replace(/^\s+|\s+$/g,'');
		}
		return label;
	};

	$rootScope.groupLabel = function(label){
		if(label.indexOf(':') > 0){
			return label.substr(0, label.indexOf(':')).replace(/^\s+|\s+$/g,'');
		}
		return $rootScope.defaultGroup;
	};

	/**
	 * Generates a Short Code for a drop, which can be inserted into text
	 */
	$rootScope.getDropShortcode = function(drop){
		if(!drop) return "";
		return "[drop " + drop._internal.guid + "]";
	};

	/**
	 * Searches the given Text for drop shortcodes and returns the
	 * found drops as an array of objects like { guid: "guid-abc", found: true|false, drop: null|drop }.
	 */
	$rootScope.getDropsFromText = function(token){
		if(typeof(token) !== 'string') return [];
		var results = [];
		var guid;
		var regexp = /\[drop ([^\]]+)\]/gi;
		while(guid = regexp.exec(token)) {
			var drop = $scope.getDropByGuid(guid[1]);
			if(drop){
				results.push({ guid: guid[1], found: true, drop: drop });
			}else{
				results.push({ guid: guid[1], found: false, drop: null });
			}
		}
		return results;
	};

	$rootScope.verifyChanges = function(){
		console.log('verfiying changes and running updates');
		// run upgrade manager
		$scope.runUpgrades(core.settings);
		// check for multilang errors
		$scope.updateMultiLangFields();
		// set groups
		$scope.updateGroups();
		// check licence errors
		$scope.checkLicenceErrors();
		// check for invalid drops
		$scope.checkDropErrors();
		// show dashboard
		$scope.showOverview();
		//  check for forced upgrades
		$scope.checkForcedUpgrades();
	};

	/**
	 * Try to Save the Settings, but make sure to check for revision conflicts
	 */
	$scope.trySaveSettings = function(){

		$scope.saveInProgress = true;
		var nextRevision = { revision: core.latestRevision.revision + 1, user: core.adminUser };

		$.ajax({
			type: 'POST',
			url: core.shopURL,
			data: {
				"kk_dropper_admin_pass": core.adminSecret,
				"kk_dropper_get_latest_revision": true
			},
			success: function(res){
				$scope.$apply(function(){
					// check for revision conflicts
					console.log("latest revision:", res, "vs:", nextRevision);
					if(nextRevision.revision <= res.revision){
						modal.show('SaveConflictModal', function(answer){
							// override changes
							if(answer === "mine"){
								$scope.saveSettings(res.revision + 1);
							// load foreign version
							}else if(answer === "yours"){
								core.fn.loadSettings().then(function(){
									core.fn.reload();
									$scope.saveInProgress = false;
									$scope.setDirtySaveState(false);
								});
							}
						}, {'conflict': res});
					}else{
						$scope.saveSettings(core.latestRevision.revision + 1);
					}
				});
			},
			error: function(err){
				message.error('Letzte Revision konnte nicht erkannt werden.', 'Speichervorgang abgebrochen!');
				$scope.saveInProgress = false;
				return;
			}
		});
	};

	/**
	 * Save the Settings via Ajax
	 */
	$scope.saveSettings = function(rev){

		$scope.saveInProgress = true;
		core.settings.dropperVersion = core.dropperVersion;
		core.latestRevision = { revision: rev, user: core.adminUser };
		core.settings.meta = core.latestRevision;

		// remove angularjs hashes from our save and get some nice json object...
		var output = angular.toJson(core.settings);

		// save it!
		$.ajax({
			type: 'POST',
			url: core.shopURL,
			data: {
				"kk_dropper_save_data": output,
				"kk_dropper_admin_pass": core.adminSecret
			},
			success : function(res){
				console.log('saved: ', res);
				$scope.$apply(function(){
					$scope.saveInProgress = false;
					$scope.setDirtySaveState(false);
					// update backup list
					$scope.listBackups();
				});
			},
			error: function(err){
				$scope.$apply(function(){
					$scope.saveInProgress = false;
				});
				message.error('Beim Speichern ist ein Fehler aufgetreten: ', err);
			}
		});
	};

	/**
	 * Run Upgrades on the given Settings data
	 */
	$scope.runUpgrades = function(data){

		// run control center upgrades
		upgrade.runDropperUpgrades(data, function(upgraded, count) {

			// error?
			if(angular.isString(upgraded)) {
				message.error('Fehler beim Dropper Upgrade', upgraded);
				return;
			}

			// successfull upgrade?
			if(count > 0 && upgraded) {
				core.settings = upgraded;
				message.success('Dropper Upgrade auf v' + upgraded.dropperVersion + ' erfolgreich abgeschlossen!', 'Bitte speichern um die Änderungen zu übernehmen!', 0);
				$scope.setDirtySaveState(true);
			}


			// run drop upgrades
			upgrade.runDropUpgrades(upgraded, function(upgraded, upgradeInfo) {
				if(upgradeInfo && upgradeInfo.length) {
					for(var i=0; i<upgradeInfo.length; i++){
						var info = upgradeInfo[i];
						if(info.error){ 
							message.error('Fehler beim Drop Upgrade', info.error);
						}
						else {
							message.successHTML('<strong>"' + (info.template && info.template.name ? info.template.name : '') + '"</strong> Drops wurden auf Version ' + info.template.version + ' aktualisiert.<br>Bitte speichern um die Änderungen zu übernehmen.', '<strong>' + info.drop._internal.name + '</strong> wurde erfolgreich aktualisiert', 0);
						}
					}
					core.settings = upgraded;
					$scope.setDirtySaveState(true);
				}
			});
		});

	};

	/**
	 * Check if we have Licencing errors and push them out
	 * to the user, to notice.
	 */
	 $scope.checkLicenceErrors = function(){
		var errors = "";

		for(var drop in core.dropLicenses){
			var licence = core.dropLicenses[drop];
			if(licence.isError === true){
				errors += '<p><strong>' + drop + '</strong>: ' + licence.reason;
				errors += '</p>';
			}
		}

		if(errors !== ""){
			errors += '<p><a class="btn btn-small btn-danger" href="mailto:info@kreativkonzentrat.de?subject=Drops%20Lizenzfehler&body=Fehlernachricht%20hier%20pasten">Fehler melden</a></p>';
			message.errorHTML("Oups, Lizenzfehler sind aufgetreten", errors);
		}
	 };

	 /**
	  * Check for Errors in Drops that are inside the List
	  */
	 $scope.checkDropErrors = function(){
		var errors = "";
		if(core.settings && core.settings.drops && core.settings.drops.length > 0){
			for(var i=0; i<core.settings.drops.length; i++){
				var drop = core.settings.drops[i];
				var template = drop._internal.dropId;

				if(!core.templates[drop._internal.dropId]){
					errors += "<p><strong>" + drop._internal.name + "</strong> inaktiv: Drop '" + template + "' konnte nicht gefunden werden oder ist korrupt.</p>";
				}
				else if(!core.dropLicenses[template] || core.dropLicenses[template].isValid == false){
					errors += "<p><strong>" + drop._internal.name + "</strong> inaktiv: Drop '" + template + "' hat keine gültige Lizenz.</p>";
				}
			}
		 }
		 if(errors !== ""){
			message.errorHTML("Einige Drops sind inaktiv!", errors);
		 }
	 };

	/**
	 * Whenever Data has changed in our settings,
	 * call this function
	 */
	$rootScope.setDirtySaveState = function(active) {
		if(active){
			$scope.saveLabel = "Speichern";
		} else {
			$scope.saveLabel = "Gespeichert";
		}
	};

	/**
	 * Adds some Icons to the Menu that are missing due to our Style
	 */
	var addJTLIcons = function(){
		if(window.jtlIconsAdded) return;
		$('div#menu').append('<a class="btn btn-menu" href="' + core.shopURL + '" target="_blank"><small><i class="icon-shopping-cart icon-white"></i> Shop</small></a>');
		$('div#menu').append('<a class="btn btn-menu" href="' + core.adminBackendURL + '" target="_blank"><small><i class="icon-home icon-white"></i> Admin</small></a>');
		window.jtlIconsAdded = true;
	};

	/**
	 * Pings the Admin Backend via Ajax to keep the Session alive
	 */
	var registerKeepSessionAlive = function(){
		if(window.keepSessionAliveRegistered) return;
		// Ping the Admin-Backend frequently to keep Session alive
		setInterval(function() {
			$.ajax({
				url: core.adminBackendURL,
				success: function() {},
				error: function() {}
			});
		}, 5*60*1000 /* Every 5 minutes */);
		window.keepSessionAliveRegistered = true;
	};

	/**
	 * Listens on CTRL+S and saves
	 */
	var addSaveHandler = function(){
		$(document).off('keydown.dropper');
		$(document).on('keydown.dropper', function(e){
			if((e.ctrlKey === true || e.metaKey === true) && e.keyCode === 83) {
				$scope.trySaveSettings();
				e.preventDefault();
				e.stopPropagation();
			}
		});
	};

	/**
	 * Make sure the User really wants to exit Dropper
	 */
	var confirmBackspaceNavigations = function(){
		if(window.confirmNavigations) return;
		window.confirmNavigations = true;
	    // http://stackoverflow.com/a/22949859/2407309
	    var backspaceIsPressed = false;
	    $(document).keydown(function(event){
	        if (event.which == 8) {
	            backspaceIsPressed = true;
	        }
	    });
	    $(document).keyup(function(event){
	        if (event.which == 8) {
	            backspaceIsPressed = false;
	        }
	    });
	    $(window).on('beforeunload', function(){
	        if (backspaceIsPressed) {
	            backspaceIsPressed = false;
	            return "Du bist dabei Dropper zu verlassen. Möchtest du das wirklich?";
	        }
	    });
	};
	
	/**
	 * Handles Tooltip and Modal activation
	 */
	$rootScope.reactivateModals = function(){
		// trigger modals to bind events again
		setTimeout(function(){
			$('.modal').modal();
			$('.trigger-tooltip').tooltip();
			$('[data-toggle="popover"]').popover();
			$('.popover').popover();
		}, 100);
	};

	addSaveHandler();
	addJTLIcons();
	registerKeepSessionAlive();
	confirmBackspaceNavigations();
	$rootScope.reactivateModals();

	// perform checks, but only if we have a valid license!
	if(core.dropperLicense.hasLicense && core.dropperLicense.isValid){
		// run verification tasks
		$rootScope.verifyChanges();
	
		core.fn.onReload(function(){
			$rootScope.verifyChanges();
		});
	}

	
	// load backups history
	$scope.listBackups();

	// watch for changes, update groups
	$scope.$watch('activeDrop._internal.name', function(){

		// update group labels
		$scope.updateGroups();

	}, false);

	var oldActiveDrop = null;
	$scope.$watch('activeDrop', function(){
		// did we have changes inside the active drop itself,
		// or did just the reference change?
		if($scope.activeDrop === oldActiveDrop){
			console.log('active drop:', $scope.activeDrop);
			$scope.setDirtySaveState(true);
		} 
		// active drop reference changed!
		else {
			for(var groupName in $scope.groups){
				if($scope.groups[groupName].indexOf($rootScope.activeDrop) >= 0){
					// activate group
					$scope.openGroup($scope.groups[groupName]);
				}
			}
			if($rootScope.activeDrop){
				$rootScope.activeTemplate = core.templates[$rootScope.activeDrop._internal.dropId];
			}else{
				$rootScope.activeTemplate = undefined;
			}
		}
		
		// update linkmaps
		if(window.linkMapBuildTimeout){
			clearTimeout(window.linkMapBuildTimeout);
		}
		window.linkMapBuildTimeout = setTimeout(function(){
			$scope.$apply(function(){
				$scope.buildLinkMap($scope.activeDrop);
			});
		}, 800);

		oldActiveDrop = $scope.activeDrop;
	}, true);
	
	/**
	 * Watch for the Drop search in the sidebar. if active, open all groups, otherwise go back to the last open group
	 */
	$scope.$watch('drop_search', function(val){
		// when we are searching we want to open all groups
		if(val && val.length){
			for(var groupName in $scope.groups){
				$scope.groups[groupName].open = true;
			}
		}
		// not search anymore... try to open the group with the active drop
		else {
			for(var groupName in $scope.groups){
				if($rootScope.activeDrop && $scope.groups[groupName].indexOf($rootScope.activeDrop) >= 0){
					// activate group
					$scope.groups[groupName].open = true;
				} else {
					$scope.groups[groupName].open = false;
				}
			}
		}
	});

	/**
	 * Watch for changes in the drops array (e.g. item added / removed)
	 */
	$scope.$watchCollection('core.settings.drops', function(newVal, oldVal){
		if(oldVal !== newVal) $scope.setDirtySaveState(true);
		$scope.updateGroups();
		$scope.buildLinkMap();
	});

	/**
	 * Watch for lang changes and set dirty state accordingly
	 */
	$scope.$watch('core.settings.lang', function(newVal, oldVal) {
		if (oldVal && newVal !== oldVal) $scope.setDirtySaveState(true);
	}, true);

	// extend the rootscope with some legacy functions that we still need to keep old drops running :(
	$.extend($rootScope, rootScopeLegacy);
	

}]);