angular.module('app').controller('DuplicateModalController', ['$modalScope', '$rootScope', 'core', 'drop', 'utils', function($modalScope, $rootScope, core, drop, utils){
	$modalScope.drop = drop;
	$modalScope.core = core;
	$modalScope.delayShow = true;
	
	$modalScope.group = {
		name: $rootScope.groupLabel(drop._internal.name)
	};
	
	if($modalScope.group.name === "Ungruppiert") $modalScope.group.name = "";
	
	var duplicateDrop = function(original){
		var newDrop = JSON.parse(angular.toJson(original));
		utils.Drops.deepReplaceGuids(newDrop);
		return newDrop;
	};

	var dropNameExists = function(cmp){
		for(var i=0; i<core.settings.drops.length; i++){
			var drop = core.settings.drops[i];
			if(drop && drop._internal && drop._internal.name === cmp){
				return true;
			}
		}
		return false;
	};
	
	var buildDependencyTree = function(drop, level, parent, cloneMap){
		var tree = [];
		cloneMap = cloneMap || {};
		
		if(!drop) return tree;

		// create clones, but only one per original
		if(!cloneMap[drop._internal.guid]){
			cloneMap[drop._internal.guid] = duplicateDrop(drop);
		}
		
		var dep = { 
			level: level,
			drop: drop,
			duplicate: cloneMap[drop._internal.guid],
			parent: (parent ? parent : null),
			clone: level == 0 ? true : false,
			children: []
		};

		// add this to the parents children collection
		if(parent) parent.children.push(dep);

		// add the dep into the tree
		tree.push(dep);

		// get a list of drops linked form this drop
		var linked = $rootScope.getDropsFromText(angular.toJson(drop));
		
		var alreadyFound = [];
		
		if(linked && linked.length){
			for(var i=0; i<linked.length; i++){
				var linkedDrop = linked[i].drop;
				if(alreadyFound.indexOf(linkedDrop._internal.guid) < 0){
					tree = tree.concat(buildDependencyTree(linkedDrop, level+1, dep, cloneMap));
					alreadyFound.push(linkedDrop._internal.guid);
				}
			}
		}
		return tree;
	};

	$modalScope.getName = function(dropDep){
		if(dropDep.clone){ 
			var clonedDropName = ($modalScope.group.name ?  $modalScope.group.name + ': ' + $rootScope.ungroupLabel(dropDep.drop._internal.name) : $rootScope.ungroupLabel(dropDep.drop._internal.name));
			return clonedDropName + (dropNameExists(clonedDropName) ? ' (Kopie)' : '');
		}
		return dropDep.drop._internal.name;
	};
	
	$modalScope.cloneAvailable = function(dropDep){
		if(dropDep.parent){
			if(!dropDep.parent.clone) return false;
			return $modalScope.cloneAvailable(dropDep.parent);
		}
		return true;
	};

	$modalScope.toggleMode = function(dropDep){
		if(dropDep.level === 0) return;

		var cloneAvailable = $modalScope.cloneAvailable(dropDep);

		if(cloneAvailable && !dropDep.clone){
			dropDep.clone = true;
		}
		else { 
			dropDep.clone = false;
			$modalScope.resetClones(dropDep);
		}
	};
	
	$modalScope.resetClones = function(dropDep){
		if(dropDep && dropDep.children && dropDep.children.length){
			for(var i=0; i<dropDep.children.length; i++){
				var child = dropDep.children[i];
				child.clone = false;
				$modalScope.resetClones(child);
			}
		}
	};
	
	var applyOperations = function(dropDep, addList){
		dropDep = dropDep || $modalScope.depTree[0];
		addList = addList || [];

		console.log(dropDep);

		// we need to replace all children links that we want to copy inside of our drop
		if(dropDep.clone){
			var duplicateJSON = angular.toJson(dropDep.duplicate);
			for(var i=0; i<dropDep.children.length; i++){
				var childDep = dropDep.children[i];
				if(childDep.clone){
					duplicateJSON = duplicateJSON.replace(new RegExp(childDep.drop._internal.guid, 'g'), childDep.duplicate._internal.guid);
					$.extend(true, dropDep.duplicate, JSON.parse(duplicateJSON));
					applyOperations(childDep, addList);
				}
			}
			
			dropDep.duplicate._internal.name = $modalScope.getName(dropDep);

			// remove insert conditions for all linked drops
			if(dropDep.level > 0){
				dropDep.duplicate._internal.selector = "";
				dropDep.duplicate._internal.replaceMethod = "";
			}

			// push it to the list, if not done already			
			if(addList.indexOf(dropDep.duplicate) < 0){
				addList.push(dropDep.duplicate);
			}
		}
		return addList;
	};
	
	$modalScope.applyOperations = function(){
		var addList = applyOperations();
		if(!addList || !addList.length) return close();

		// add them in reverse order, so that the original order is being kept
		for(var i=addList.length-1; i>=0; i--){
			core.settings.drops.push(addList[i]);
		}
		
		console.log('addList:', addList);
		$rootScope.activateDrop(addList[addList.length-1]);
		$modalScope.close();
	};

	// build the dependency tree
	$modalScope.depTree = buildDependencyTree(drop, 0);

	// do a simply clone if we got only one dependency (drop to clone contains no links to other drops)
	if($modalScope.depTree.length === 1){
		$modalScope.applyOperations();
		$modalScope.close();
	} else {
		$modalScope.show();
	}
}]);