angular.module('app').controller('ImportExportModalController', ['$modalScope', '$rootScope', '$element', '$filter','core', 'message', 'type', 'core', function($modalScope, $rootScope, $element, $filter, core, message, type, core){
	$modalScope.tab = type;
	$modalScope.core = core;

	$modalScope.exportDrops = {
		items: []
	};

	$modalScope.exportOption = {
		"description" : "",
		"optionLabel" : "_internal.name",
		"optionValue" : "_internal.guid",
		"options" : core.settings.drops
	}

	$modalScope.importOption = {
		"description" : "",
		"options" : [
			{"label": "Ja", "value": true},
			{"label": "Nein", "value": false}
		]
	}

	$modalScope.importLangOption = {
		"description" : "",
		"options" : [
			{"label": "Nicht importieren", "value": "none"},
			{"label": "Eigene beibehalten", "value": "gracefull"},
			{"label": "Eigene überschreiben", "value": "override"}
		]
	}

	$modalScope.importDrops = {
		drops: [],
		timestamp: null,
		selected: [],
		replace: false,
		lang: {},
		langOption: "none"
	};

	jUploader = $element.find('.import-uploader');

	//////////////////////////
	// EXPORT
	//////////////////////////

	$modalScope.setTab = function(tab){
		$modalScope.tab = tab;
	}

	$modalScope.selectAllForExport = function(){
		var res = [];
		for(var i=0; i<core.settings.drops.length; i++){
			var guid = core.settings.drops[i]._internal.guid;
			res.push(guid);
		}
		$modalScope.exportDrops.items = res;
	}

	$modalScope.selectNoneForExport = function(){
		$modalScope.exportDrops.items = [];
	}

	$modalScope.doExport = function(){
		// do nothing on empty
		if($modalScope.exportDrops.items.length === 0) return;

		// resolve the drops
		var res = [];
		var lang = {};

		for(var i=0; i<$modalScope.exportDrops.items.length; i++){
			var guid = $modalScope.exportDrops.items[i];
			var drop = $rootScope.getDropByGuid(guid);
			if(!drop) continue;
			res.push($.extend(true, {}, drop));
			// does the drop have lang settings?
			if(core.settings.lang && core.settings.lang[drop._internal.dropId]){
				lang[drop._internal.dropId] = $.extend(true, {}, core.settings.lang[drop._internal.dropId]);
			}
		}

		var data = angular.toJson({ 
			drops: res, 
			timestamp: new Date().getTime(),
			dropperVersion: core.dropperVersion,
			lang: lang,
			user: core.adminUser
		}, false);

		var filename = "drops.json";
		var blob = new Blob([data], {type: "application/json;charset=utf-8"});
		saveAs(blob, filename);
	}

	//////////////////////////
	// IMPORT
	//////////////////////////

	$modalScope.showUploader = function(){
		jUploader.click();
	}


	// read the file when there is one selected!
	jUploader.change(function(){
		var file = $(this).get(0).files[0];

		if(file){
			var reader = new FileReader();
			reader.readAsText(file, "UTF-8");
			reader.onload = function(e) {
				try{
					var data = angular.fromJson(e.target.result);
					$modalScope.$apply(function(){
						$modalScope.setImportData(data);
					});
				}catch(ex){
					message.error("Backup konnte nicht geladen werden!", ex.message);
				}
			}
			reader.onerror = function(e){
				message.error("Backup konnte nicht gelesen werden!", "Eventuell ist die Datei korrupt");
			}
		}
	});

	$modalScope.setImportData = function(data){
		if(data.drops){
			var included = [];
			for(var i=0; i<data.drops.length; i++){
				var drop = data.drops[i];
				var state = $modalScope.getImportState(drop);
				if($modalScope.canImport(drop) && state !== "same"){
					included.push(drop);
				}
			}

			$modalScope.importDrops.timestamp = data.timestamp;
			$modalScope.importDrops.user = data.user;
			$modalScope.importDrops.drops = data.drops;
			$modalScope.importDrops.lang = data.lang || {};
			$modalScope.importDrops.selected = included;
		}
	}

	$modalScope.importToggle = function(drop){
		var idx = $modalScope.importDrops.selected.indexOf(drop);
		if(idx >= 0){
			// drop is selected, remove it from the list
			$modalScope.importDrops.selected.splice(idx, 1);
		}else{
			$modalScope.importDrops.selected.push(drop);
		}
	}

	$modalScope.isImportSelected = function(drop){
		return ($modalScope.importDrops.selected.indexOf(drop) >= 0);
	}

	$modalScope.getImportState = function(drop){
		var other = $rootScope.getDropByGuid(drop._internal.guid);
		// new one?
		if(!other){
			return "new";
		}
		// not new? check if it is the same!
		var otherJSON = angular.toJson(other);
		var dropJSON = angular.toJson(drop);

		return (otherJSON === dropJSON ? "same" : "update");
	}

	$modalScope.canImport = function(drop){
		var dropId = drop._internal.dropId;
		return $rootScope.canAddDrop(dropId);
	}

	$modalScope.getImportDate = function(){
		var date = new Date($modalScope.importDrops.timestamp);
		return $filter('date')(date, 'dd. MMMM, HH:mm');
	}

	$modalScope.selectAllForImport = function(){
		var res = [];
		for(var i=0; i<$modalScope.importDrops.drops.length; i++){
			var drop = $modalScope.importDrops.drops[i];
			res.push(drop);
		}
		$modalScope.importDrops.selected = res;
	}

	$modalScope.selectNoneForImport = function(){
		$modalScope.importDrops.selected = [];
	}

	$modalScope.doImport = function(){
		if($modalScope.importDrops.selected.length === 0) return;
		
		var replacedItems = 0;
		var newItems = 0;

		// empty the drops from settings
		if($modalScope.importDrops.replace === true){
			core.settings.drops = [];
		}

		for(var i=0; i<$modalScope.importDrops.selected.length; i++){
			var importDrop = $modalScope.importDrops.selected[i];
			var importTemplate = importDrop._internal.dropId;

			var orig = $rootScope.getDropByGuid(importDrop._internal.guid);
			if(orig){
				// replace it!
				var origIdx = core.settings.drops.indexOf(orig);
				if(origIdx >= 0){
					core.settings.drops[origIdx] = importDrop;
					replacedItems ++;
				}
			}else{
				// add it to the list
				core.settings.drops.push(importDrop);
				newItems ++;
			}
			// merge lang setting
			if($modalScope.importDrops.langOption !== "none" && $modalScope.importDrops.lang && $modalScope.importDrops.lang[importTemplate]){
				if(!core.settings.lang) core.settings.lang = {};
				if(!core.settings.lang[importTemplate]) core.settings.lang[importTemplate] = {};

				switch($modalScope.importDrops.langOption){
					case "override":
						core.settings.lang[importTemplate] = $.extend(true, core.settings.lang[importTemplate], $modalScope.importDrops.lang[importTemplate]);
					break;
					case "gracefull":
						core.settings.lang[importTemplate] = $.extend(true, $modalScope.importDrops.lang[importTemplate], core.settings.lang[importTemplate]);
					break;
				}
			}
		}
		if(replacedItems > 0 || newItems > 0){
			$rootScope.setDirtySaveState(true);
			$rootScope.verifyChanges();
		}
		message.success('Import abgeschlossen', 'Es wurden ' + replacedItems + ' Drops durch die importierte Version ersetzt und ' + newItems + ' neue Drops hinzugefügt', 6000);
		$modalScope.close();
	}
}]);

/*! FileSaver.js
 *  A saveAs() FileSaver implementation.
 *  2014-01-24
 *
 *  By Eli Grey, http://eligrey.com
 *  License: X11/MIT
 *    See LICENSE.md
 */

/*global self */
/*jslint bitwise: true, indent: 4, laxbreak: true, laxcomma: true, smarttabs: true, plusplus: true */

/*! @source http://purl.eligrey.com/github/FileSaver.js/blob/master/FileSaver.js */

var saveAs = saveAs
  // IE 10+ (native saveAs)
  || (typeof navigator !== "undefined" &&
	  navigator.msSaveOrOpenBlob && navigator.msSaveOrOpenBlob.bind(navigator))
  // Everyone else
  || (function(view) {
	"use strict";
	// IE <10 is explicitly unsupported
	if (typeof navigator !== "undefined" &&
		/MSIE [1-9]\./.test(navigator.userAgent)) {
		return;
	}
	var
		  doc = view.document
		  // only get URL when necessary in case BlobBuilder.js hasn't overridden it yet
		, get_URL = function() {
			return view.URL || view.webkitURL || view;
		}
		, URL = view.URL || view.webkitURL || view
		, save_link = doc.createElementNS("http://www.w3.org/1999/xhtml", "a")
		, can_use_save_link = !view.externalHost && "download" in save_link
		, click = function(node) {
			var event = doc.createEvent("MouseEvents");
			event.initMouseEvent(
				"click", true, false, view, 0, 0, 0, 0, 0
				, false, false, false, false, 0, null
			);
			node.dispatchEvent(event);
		}
		, webkit_req_fs = view.webkitRequestFileSystem
		, req_fs = view.requestFileSystem || webkit_req_fs || view.mozRequestFileSystem
		, throw_outside = function(ex) {
			(view.setImmediate || view.setTimeout)(function() {
				throw ex;
			}, 0);
		}
		, force_saveable_type = "application/octet-stream"
		, fs_min_size = 0
		, deletion_queue = []
		, process_deletion_queue = function() {
			var i = deletion_queue.length;
			while (i--) {
				var file = deletion_queue[i];
				if (typeof file === "string") { // file is an object URL
					URL.revokeObjectURL(file);
				} else { // file is a File
					file.remove();
				}
			}
			deletion_queue.length = 0; // clear queue
		}
		, dispatch = function(filesaver, event_types, event) {
			event_types = [].concat(event_types);
			var i = event_types.length;
			while (i--) {
				var listener = filesaver["on" + event_types[i]];
				if (typeof listener === "function") {
					try {
						listener.call(filesaver, event || filesaver);
					} catch (ex) {
						throw_outside(ex);
					}
				}
			}
		}
		, FileSaver = function(blob, name) {
			// First try a.download, then web filesystem, then object URLs
			var
				  filesaver = this
				, type = blob.type
				, blob_changed = false
				, object_url
				, target_view
				, get_object_url = function() {
					var object_url = get_URL().createObjectURL(blob);
					deletion_queue.push(object_url);
					return object_url;
				}
				, dispatch_all = function() {
					dispatch(filesaver, "writestart progress write writeend".split(" "));
				}
				// on any filesys errors revert to saving with object URLs
				, fs_error = function() {
					// don't create more object URLs than needed
					if (blob_changed || !object_url) {
						object_url = get_object_url(blob);
					}
					if (target_view) {
						target_view.location.href = object_url;
					} else {
						window.open(object_url, "_blank");
					}
					filesaver.readyState = filesaver.DONE;
					dispatch_all();
				}
				, abortable = function(func) {
					return function() {
						if (filesaver.readyState !== filesaver.DONE) {
							return func.apply(this, arguments);
						}
					};
				}
				, create_if_not_found = {create: true, exclusive: false}
				, slice
			;
			filesaver.readyState = filesaver.INIT;
			if (!name) {
				name = "download";
			}
			if (can_use_save_link) {
				object_url = get_object_url(blob);
				// FF for Android has a nasty garbage collection mechanism
				// that turns all objects that are not pure javascript into 'deadObject'
				// this means `doc` and `save_link` are unusable and need to be recreated
				// `view` is usable though:
				doc = view.document;
				save_link = doc.createElementNS("http://www.w3.org/1999/xhtml", "a");
				save_link.href = object_url;
				save_link.download = name;
				var event = doc.createEvent("MouseEvents");
				event.initMouseEvent(
					"click", true, false, view, 0, 0, 0, 0, 0
					, false, false, false, false, 0, null
				);
				save_link.dispatchEvent(event);
				filesaver.readyState = filesaver.DONE;
				dispatch_all();
				return;
			}
			// Object and web filesystem URLs have a problem saving in Google Chrome when
			// viewed in a tab, so I force save with application/octet-stream
			// http://code.google.com/p/chromium/issues/detail?id=91158
			if (view.chrome && type && type !== force_saveable_type) {
				slice = blob.slice || blob.webkitSlice;
				blob = slice.call(blob, 0, blob.size, force_saveable_type);
				blob_changed = true;
			}
			// Since I can't be sure that the guessed media type will trigger a download
			// in WebKit, I append .download to the filename.
			// https://bugs.webkit.org/show_bug.cgi?id=65440
			if (webkit_req_fs && name !== "download") {
				name += ".download";
			}
			if (type === force_saveable_type || webkit_req_fs) {
				target_view = view;
			}
			if (!req_fs) {
				fs_error();
				return;
			}
			fs_min_size += blob.size;
			req_fs(view.TEMPORARY, fs_min_size, abortable(function(fs) {
				fs.root.getDirectory("saved", create_if_not_found, abortable(function(dir) {
					var save = function() {
						dir.getFile(name, create_if_not_found, abortable(function(file) {
							file.createWriter(abortable(function(writer) {
								writer.onwriteend = function(event) {
									target_view.location.href = file.toURL();
									deletion_queue.push(file);
									filesaver.readyState = filesaver.DONE;
									dispatch(filesaver, "writeend", event);
								};
								writer.onerror = function() {
									var error = writer.error;
									if (error.code !== error.ABORT_ERR) {
										fs_error();
									}
								};
								"writestart progress write abort".split(" ").forEach(function(event) {
									writer["on" + event] = filesaver["on" + event];
								});
								writer.write(blob);
								filesaver.abort = function() {
									writer.abort();
									filesaver.readyState = filesaver.DONE;
								};
								filesaver.readyState = filesaver.WRITING;
							}), fs_error);
						}), fs_error);
					};
					dir.getFile(name, {create: false}, abortable(function(file) {
						// delete file if it already exists
						file.remove();
						save();
					}), abortable(function(ex) {
						if (ex.code === ex.NOT_FOUND_ERR) {
							save();
						} else {
							fs_error();
						}
					}));
				}), fs_error);
			}), fs_error);
		}
		, FS_proto = FileSaver.prototype
		, saveAs = function(blob, name) {
			return new FileSaver(blob, name);
		}
	;
	FS_proto.abort = function() {
		var filesaver = this;
		filesaver.readyState = filesaver.DONE;
		dispatch(filesaver, "abort");
	};
	FS_proto.readyState = FS_proto.INIT = 0;
	FS_proto.WRITING = 1;
	FS_proto.DONE = 2;

	FS_proto.error =
	FS_proto.onwritestart =
	FS_proto.onprogress =
	FS_proto.onwrite =
	FS_proto.onabort =
	FS_proto.onerror =
	FS_proto.onwriteend =
		null;

	view.addEventListener("unload", process_deletion_queue, false);
	saveAs.unload = function() {
		process_deletion_queue();
		view.removeEventListener("unload", process_deletion_queue, false);
	};
	return saveAs;
}(
	   typeof self !== "undefined" && self
	|| typeof window !== "undefined" && window
	|| this.content
));
// `self` is undefined in Firefox for Android content script context
// while `this` is nsIContentFrameMessageManager
// with an attribute `content` that corresponds to the window

if (typeof module !== "undefined" && module !== null) {
  module.exports = saveAs;
} else if ((typeof define !== "undefined" && define !== null) && (define.amd != null)) {
  define([], function() {
	return saveAs;
  });
}