angular.module('app').controller('InsertConditionsModalController', ['$modalScope', '$rootScope', '$element', '$timeout', 'core', 'utils', 'drop', function($modalScope, $rootScope, $element, $timeout, core, utils, drop){

	$modalScope.pickerActive = false;
	$modalScope.pickerSelectionMode = true;

	///////////////////////////////////////////////
	// INSERT CONDITIONS
	///////////////////////////////////////////////
	$modalScope.root = $rootScope;
	$modalScope.drop = $.extend(true, {}, drop);
	$modalScope.insertConditionConfig = [
		//{name: 'Selektor', description: 'Ein Selektor der die Position des Drops innerhalb des Templates angibt.', type: 'text', property: 'selector'},
		{name: 'Selektor', description: 'Ein Selektor der die Position des Drops innerhalb des Templates angibt.', type: 'selector-input', property: 'selector'},
		{name: 'Einfügemethode', description: 'Bestimme wie das Drop an der durch den Selektor ausgewählten Stelle eingefügt werden soll.', type: 'imagepicker', optionImageRoot: 'admin', property: 'replaceMethod', width: 375, options: [
			{value: 'before', label: 'Vorstellen', detail: 'Platziert das Drop vor dem ausgewählten Element', image: 'img/insert-method-before.png'},
			{value: 'after', label: 'Anstellen', detail: 'Platziert das Drop nach dem ausgewählten Element', image: 'img/insert-method-after.png'}, 
			{value: 'prepend', label: 'Vorhängen', detail: 'Hängt das Drop als ersten Inhalt in das ausgewählte Element ein', image: 'img/insert-method-prepend.png'}, 
			{value: 'append', label: 'Anhängen', detail: 'Hängt das Drop als letzten Inhalt an das ausgewählte Element an', image: 'img/insert-method-append.png'}, 
			{value: 'html', label: 'Inhalt Ersetzen', detail: 'Ersetzt den Inhalt des ausgewählten Elementes durch den Inhalt des Drops', image: 'img/insert-method-html.png'},
			{value: 'replaceWith', label: 'Ersetzen', detail: 'Ersetzt das ausgewählte Element komplett durch den Inhalt des Drops', image: 'img/insert-method-replaceWith.png'},
			{value: '', label: 'Keine', image: 'img/insert-method-none.png'}
			]
		}
	];
	
	/**
	 * Generate a Smarty Expression out of the set Conditionals on the active Drop
	 */
	$modalScope.generateSmartyExpression = function(drop){
		if(drop._internal) drop._internal.condition = utils.conditionToSmartyExpression(drop._internal.conditionalSettings);
	};

	var dropGuid = $modalScope.drop._internal.guid;

	///////////////////////////////////////////////
	// SELECTOR PICKER
	///////////////////////////////////////////////

	var setPickerParam = function(url) {
		url  = url.replace(/(?:\?|&)zonePicker(?:=[^&]+)*/gi, '');
		var parts = url.split('#');
		parts[0] = parts[0] + (parts[0].indexOf('?') >= 0 ? '&zonePicker=' : '?zonePicker=') + dropGuid;
		
		return parts.join('#');
	};

	$modalScope.initPickerOptions = function() {

		// initialize picking options
		var initialUrl = 	$modalScope.drop._internal.selectorUrl
							? core.shopURL + $modalScope.drop._internal.selectorUrl 
							: core.shopURL;

		$modalScope.selectorOption = {
			selector: $modalScope.drop._internal.selector,
			replaceMethod: $modalScope.drop._internal.replaceMethod,
			visible: false,
			frameUrl: setPickerParam(initialUrl),
			inputUrl: initialUrl
		};

	};

	$modalScope.setPickerVisible = function(state){
		$modalScope.pickerActive = state;
	};

	$modalScope.togglePickerVisibility = function(){
		$modalScope.pickerActive = !$modalScope.pickerActive;
	};

	$modalScope.setReplaceMethod = function(method){
		$modalScope.selectorOption.replaceMethod = method;
	};

	// when the picker is getting activated, we want to load the
	// iframe contents, but not earlier...
	$modalScope.$watch('pickerActive', function(active) {
		
		var updateUrl = function(url, fWindow) {
			console.log('updating url', url);

			$modalScope.$apply(function(){
				 
				// if no zonePicker param given, lets rewrite the url
				var paramUrl = setPickerParam(url);
				if (paramUrl !== url) fWindow.location.href = paramUrl;
	
				// set the url (replaces the zonePicker Param)
				$modalScope.selectorOption.inputUrl = url.replace(/(?:\?|&)zonePicker(?:=[^&]+)*/gi, '');  //url.replace('?zonePicker', '').replace('&zonePicker', '');
				$modalScope.frameLoading = true;
			});
		};

		if (active) {

			$modalScope.initPickerOptions();

			$timeout(function() {

				// set us loading for the first time the iframe is shown
				$modalScope.frameLoading = true;

				// get the iframe and window
				var frame = $element.find('iframe.picker-frame');
				var fWindow = frame.get(0).contentWindow || frame.get(0);
	
				// when the src changes, initialize us
				frame.load(function(){
					updateUrl(fWindow.location.href, fWindow);

					// loaded!
					$modalScope.$apply(function(){
						$modalScope.frameLoading = false;	
					});
					
					// if we have already a selector chosen, tell the iframe to select it
					if ($modalScope.drop && $modalScope.drop._internal && $modalScope.drop._internal.selector) {
						var selector = $modalScope.drop._internal.selector.split(',')[0].trim();
						var zoneInfo = core.templateZones[selector.replace('@', '').replace(/\(\d+\)/g, '($index)')] || null;

						fWindow.postMessage({
							type: 'kk_dropper.presetResult',
							zone: (zoneInfo && zoneInfo.zone ? zoneInfo.zone : null),
							selector: selector,
							replaceMethod: $modalScope.drop._internal.replaceMethod
						}, '*');
					}

					// when the url is changed we want to know this
					fWindow.onbeforeunload = function(evt) {
						// loading new location!
						$modalScope.$apply(function(){
							$modalScope.frameLoading = false;	
						});
						if (fWindow.document.activeElement && fWindow.document.activeElement.href) updateUrl(fWindow.document.activeElement.href, fWindow);
					};
				});
				
			});
		}
	});


	// check for iframe messages
	window.onmessage = function(msg) {
		if (!msg || !msg.data || !msg.data.type) return;

		// we got a result from the picker, let's take it
		if (msg.data.type === 'kk_dropper.pickerResult') {
			
			$modalScope.$apply(function() {
				$modalScope.selectorOption.selector = msg.data.selector;
				$modalScope.selectorOption.replaceMethod = msg.data.method;
				$modalScope.applySelectorChanges();
			});
		}
		
		// selection mode was changed from within the frame
		if (msg.data.type === 'kk_dropper.selectionMode') {
			$modalScope.$apply(function() {
				$modalScope.pickerSelectionMode = msg.data.active;
			});
		}
	};

	$modalScope.updatePickerUrl = function(){
		$modalScope.selectorOption.frameUrl = setPickerParam($modalScope.selectorOption.inputUrl);
	};
	
	// this will inform our iframe if selection mode is active or not
	$modalScope.toggleSelectionMode = function() {

		$modalScope.pickerSelectionMode = !$modalScope.pickerSelectionMode;

		// get the iframe
		var frame = $element.find('iframe.picker-frame');
		var win = frame.get(0).contentWindow || frame.get(0);
		win.postMessage({
			type: 'kk_dropper.selectionMode',
			active: $modalScope.pickerSelectionMode
		}, '*');
	};

	// apply changes
	$modalScope.applySelectorChanges = function(){
		$modalScope.drop._internal = $.extend(true, {}, $modalScope.drop._internal, {
			selector: $modalScope.selectorOption.selector,
			replaceMethod: $modalScope.selectorOption.replaceMethod,
			selectorUrl: $modalScope.selectorOption.inputUrl.replace(core.shopURL, "")
		});

		$modalScope.pickerActive = false;
	};

	
}]);
