angular.module('app').controller('SettingsModalController', ['$modalScope', '$rootScope', 'core', function($modalScope, $rootScope, core){

	$modalScope.core = core;
	var settingsChanged = false;

	// bindings to what we can modify
	$modalScope.settings = {
		integration: core.settings.integration || "_auto",
		backupLifetime: core.settings.backupLifetime,
		deferJS: core.settings.deferJS || false,
		inlineLimit: core.settings.inlineLimit || "20",
		inlineTypes: core.settings.inlineTypes || []
	};
	
	// detect setting changes to set the save button to dirty state later on
	$modalScope.$watch('settings', function(newval, oldval) {
		if (angular.toJson(newval) !== angular.toJson(oldval)) {
			settingsChanged = true;
		}
	}, true);
	
	// setting property grid
	$modalScope.settingsConfig = [
		{
			name: "Templateintegration",
			description: "Auf welchem Template basiert das von dir eingesetzte Shop-Template?",
			type: "select",
			options: "TemplateIntegrations",
			section: "Allgemein",
			property: "integration"
		},
		{
			name: "Ressourcen einbetten?",
			description: "Erlabut das einbetten bestimmter Javascript und CSS Dateien direkt in den Quellcode der Seite, um Server-Anfragen zu sparen",
			type: "multiselect",
			options: [
				{ label: "CSS Dateien (.css)", value: "css" },
				{ label: "Javascript Dateien (.js)", value: "js" }
			],
			section: "Ressourcen",
			property: "inlineTypes"
		},
		{
			name: "Ressourcen einbetten bis Dateigrößen von...",
			description: "Setze eine Obergrenze der Dateigröße in Kilobytes, bis zu welcher diese Dateien direkt in dem DOM geschrieben werden. Der Wert '0' deaktiviert das Einbetten von Ressourcen.",
			type: "text",
			append: "KB",
			section: "Ressourcen",
			condition: "self.inlineTypes && self.inlineTypes.length",
			property: "inlineLimit"
		},
		{
			name: "Javascript verspätet laden?",
			description: "Lädt Javascript am Ende des body Elements, was zu einem schnelleren Seitenaufbau führen kann",
			type: "switch",
			options: [
				{ "label": "ja", "value": true },
				{ "label": "nein", "value": false }
			],
			section: "Ressourcen",
			property: "deferJS"
		},
		{
			name: "Aufbewahrungsdauer von Backups (Tage)",
			description: "Wieviele Tage sollen Backups von Speicherpunkten aufbewahrt werden?",
			type: "text",
			append: "Tage",
			section: "Backups",
			property: "backupLifetime"
		}
	];

	// confirmation
	$modalScope.apply = function(){
		console.log($modalScope.settings);
		$.extend(core.settings, $modalScope.settings);
		if (settingsChanged) {
			$rootScope.setDirtySaveState(true);
		}
		$modalScope.close();
	};
}]);