conditional['ArticleData'] = {
	description: 'Erlaubt das Filtern über bestimmte Eigenschaften eines Artikels (Artikeldetail)',
	name: 'Artikeleigenschaft',

	section: "Article",
	sort: 20,
	
	config: [
		{
			name: 'Eigenschaft',
			description: 'Um welche Artikeleigenschaft geht es?',
			type: 'select',
			options: [
				{ label: 'Hat Merkmalwert gesetzt', value: 'property' },
				{ label: 'Hat Hersteller gesetzt', value: 'vendor' },
				{ label: 'Hat Funktionsattribut mit Wert', value: 'attribute' },
				{ label: 'Liegt in Kategorie', value: 'category' },
				{ label: 'Hat Vaterartikel', value: 'parent' }
			],
			property: 'type'
		},
		{
			name: 'Merkmalwert',
			description: 'Merkmalwert den der Artikel gesetzt haben muss',
			type: 'select',
			options: 'ArticlePropertyValues',
			condition: 'drop.type === "property"',
			property: 'property'
		},
		{
			name: 'Hersteller',
			description: 'Hersteller des Artikels',
			type: 'select',
			options: 'Brands',
			condition: 'drop.type === "vendor"',
			property: 'vendor'
		},
		{
			name: 'Attributname',
			description: 'Name des Funktionsattributs',
			type: 'text',
			property: 'attributename',
			condition: 'drop.type === "attribute"'
		},
		{
			name: 'Attributwert',
			description: 'Wert des Funktionsattributs',
			type: 'text',
			property: 'attributevalue',
			condition: 'drop.type === "attribute"'
		},
		{
			name: 'Kategorie',
			description: 'Kategorie in welcher der angegebene Artikel liegen muss (einschliesslich Unterkategorien)',
			type: 'select',
			options: 'Categories',
			property: 'category',
			condition: 'drop.type === "category"'
		},
		{
			name: 'Kategoriemodus',
			description: 'Wähle wie genau gesucht wird',
			type: 'switch',
			options: [
				{ label: 'Genau diese Kategorie', value: 'exact' },
				{ label: 'Auch Unterkategorien', value: 'subcats'}
			],
			property: 'categorymode',
			condition: 'drop.type === "category"'
		},
		{
			name: 'Vaterartikel',
			description: 'Wähle aus, welchen Vaterartikel der Artikel haben muss',
			type: 'select',
			options: 'Articles',
			property: 'parent',
			condition: 'drop.type === "parent"'
		}
	],
	evaluateToSmarty: function(filter){
		if(!filter.type) return false;
		switch(filter.type){
			case 'vendor':
				return 'isset($Artikel->kHersteller) and $Artikel->kHersteller eq ' + filter.vendor;
			break;

			case 'property':
				return 'isset($Artikel) and $Artikel|has_propertyvalue:' + filter.property;
			break;

			case 'attribute':
				return '!empty($Artikel->FunktionsAttribute) and $Artikel|get_attribute_value:\'' + filter.attributename + '\' == "' + filter.attributevalue + '"';
			break;

			case 'category':
				return '!empty($Artikel) and $Artikel|in_category:' + filter.category + (filter.categorymode === 'subcats' ? ':true' : ':false');
			break;

			case 'parent':
				return '!empty($Artikel->kVaterArtikel) and $Artikel->kVaterArtikel == ' + filter.parent;
			break;
		}
		return false;
	}
}